@@ -1,10 +1,10 @@
 /*===========================================================================
- 
- Copyright (C) 2006-2012 Yves Renard, Julien Pommier, Jeremie Lasry.
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2006-2015 Yves Renard, Julien Pommier, Jeremie Lasry.
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -16,7 +16,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
 ===========================================================================*/
 
 #include "crack_bilaplacian.h"
@@ -240,7 +240,7 @@ gmm::resize( H, mf_mortar_deriv.nb_dof(), mf_u().nb_dof());
       /* other version of the integral matching.
        * version 2 :
        *     \int_Gamma        (u-v) \lambda  = 0, for all \lambda in \Lambda
-       *     \int_Gamma \nabla (u-v).\mu      = 0, for all \mu in M    (be carefull : \mu is vectorial.
+       *     \int_Gamma \nabla (u-v).\mu      = 0, for all \mu in M    (be careful : \mu is vectorial.
        * version 3 : only second constraint is different.   
        *     \int_Gamma \partial_n (u-v)\mu  = 0, for all \mu in M
       */
@@ -656,9 +656,9 @@ void bilaplacian_crack_problem::init_mixed_elements(void) {
   int dv = int(PARAM.int_value("DIRICHLET_VERSION", "Dirichlet version"));
   int mv = int(PARAM.int_value("MORTAR_VERSION", "Mortar version"));
   int cv = int(PARAM.int_value("CLOSING_VERSION"));  
-  dirichlet_version = getfem::constraints_type(dv);
-  mortar_version = getfem::constraints_type(mv);
-  closing_version = getfem::constraints_type(cv);
+  dirichlet_version = size_type(dv);
+  mortar_version = size_type(mv);
+  closing_version = size_type(cv);
   datafilename=PARAM.string_value("ROOTFILENAME","Base name of data files.");
   residual=PARAM.real_value("RESIDUAL"); if (residual == 0.) residual = 1e-10;
   KL = PARAM.int_value("KL", "Kirchhoff-Love model or not") != 0;
