@@ -1,10 +1,10 @@
 /*===========================================================================
- 
- Copyright (C) 2002-2012 Yves Renard, Julien Pommier.
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2002-2015 Yves Renard, Julien Pommier.
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -16,7 +16,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
 ===========================================================================*/
 
 /*
@@ -39,23 +39,23 @@
 #include "getfem/bgeot_mesh_structure.h"
 #include "getfem/getfem_config.h"
 
-using std::endl; using std::cout; using std::cerr;
-using std::ends; using std::cin;
-
-
+#if GETFEM_HAVE_METIS_OLD_API
 extern "C" void METIS_PartGraphKway(int *, int *, int *, int *, int *, int *,
-			    int *, int *, int *, int *, int *);
+                                    int *, int *, int *, int *, int *);
 extern "C" void METIS_PartGraphRecursive(int *, int *, int *, int *, int *, int *,
-			    int *, int *, int *, int *, int *);
-
-
+                                         int *, int *, int *, int *, int *);
 extern "C" void METIS_mCPartGraphKway(int *, int *, int *, int *, int *, int *, int *,
-				      int *, int *, float *, int *, int *, int *);
+                                      int *, int *, float *, int *, int *, int *);
 extern "C" void METIS_mCPartGraphRecursive(int *, int *, int *, int *, int *, int *, int *,
-				      int *, int *, int *, int *, int *);
+                                           int *, int *, int *, int *, int *);
+#elif GETFEM_HAVE_METIS
+# include <metis.h>
+#endif
+
+using std::endl; using std::cout;
 
 
-/* some Getfem++ types that we will be using */
+/* some GetFEM++ types that we will be using */
 using bgeot::base_small_vector; /* special class for small (dim<16) vectors */
 using bgeot::base_vector;
 using bgeot::base_node;  /* geometrical nodes(derived from base_small_vector)*/
@@ -324,7 +324,7 @@ public:
     sizes_.resize(1); sizes_[0] = short_type(N);
     mf.extend_vector(U_, U);
   }
-  const bgeot::multi_index &sizes() const {  return sizes_; }
+  const bgeot::multi_index &sizes(size_type) const {  return sizes_; }
   virtual void compute(getfem::fem_interpolation_context& ctx,
 		       bgeot::base_tensor &t) {
     size_type cv = ctx.convex_num();
@@ -409,7 +409,11 @@ void asm_patch_vector
 template<class MAT>
 void asm_stabilization_patch_term
 (const MAT &RM_, const getfem::mesh &mesh, const getfem::mesh_im &mimbounddown, const getfem::mesh_fem &mf_mult,
- scalar_type ratio_size, scalar_type h ){
+ scalar_type ratio_size, scalar_type
+#ifdef GETFEM_HAVE_METIS
+h
+#endif
+){
   MAT &M1 = const_cast<MAT &>(RM_);
   
   /****************************************************/
@@ -461,31 +465,44 @@ void asm_stabilization_patch_term
   }
   
   xadj[j] = k;
-  std::vector<int> adjwgt(k);
   // cout<<"xadj="<<xadj<<endl;
   //cout<<"adjncy="<<adjncy<<endl;
   //cout<<"vwgt="<<vwgt<<endl;
   
   cout<<"ratio size beween mesh and coarse mesh= "<< ratio_size <<endl;
-  
-  int wgtflag = 2, edgecut, nparts=int(size_of_crack/(ratio_size*h)), numflag = 0;
-      // float ubvec[1] = {1.03f};
-  int  options[5] = {0,0,0,0,0};
+
+  int nparts = 1;
+#if GETFEM_HAVE_METIS
+  nparts = int(size_of_crack/(ratio_size*h));
+# ifdef GETFEM_HAVE_METIS_OLD_API
+  std::vector<int> adjwgt(k); // actually Metis would also accept NULL instead of an empty array
+  int wgtflag = 2, numflag = 0, edgecut;
+  // float ubvec[1] = {1.03f};
+  int options[5] = {0,0,0,0,0};
   //METIS_mCPartGraphKway(&ne, &ncon, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), &(adjwgt[0]), &wgtflag,
   //		    &numflag, &nparts, &(ubvec[0]),  options, &edgecut, &(part[0]));
-  // METIS_mCPartGraphRecursive(&ne, &ncon, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), &(adjwgt[0]), &wgtflag,
+  //METIS_mCPartGraphRecursive(&ne, &ncon, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), &(adjwgt[0]), &wgtflag,
   //			 &numflag, &nparts,  options, &edgecut, &(part[0]));
   //METIS_PartGraphKway(&ne, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), &(adjwgt[0]), &wgtflag,
   //	  &numflag, &nparts, options, &edgecut, &(part[0]));
   METIS_PartGraphRecursive(&ne, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), &(adjwgt[0]), &wgtflag,
-			   &numflag, &nparts, options, &edgecut, &(part[0]));
-  
+                           &numflag, &nparts, options, &edgecut, &(part[0]));
+# else
+  int ncon = 1, edgecut;
+  int options[METIS_NOPTIONS] = { 0 };
+  METIS_SetDefaultOptions(options);
+  METIS_PartGraphRecursive(&ne, &ncon, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), 0, 0,
+                           &nparts, 0, 0, options, &edgecut, &(part[0]));
+# endif
   //cout<<"size_of_mesh="<<h<<endl;
   //cout<<"size_of_crack="<< size_of_crack <<endl;
   cout<<"nb_partition="<<nparts<<endl;
   // cout<<"partition="<<part<<endl;
   //cout<<"edgecut="<<edgecut<<endl;
-  
+#else
+  GMM_ASSERT1(false, "METIS not linked");
+#endif
+
 
   /**************************************************************/
   /*        Assembly matrices                                   */
@@ -861,7 +878,6 @@ int main(int argc, char *argv[]) {
       }
 
       xadj[j] = k;
-      std::vector<int> adjwgt(k);
       // cout<<"xadj="<<xadj<<endl;
       //cout<<"adjncy="<<adjncy<<endl;
       //cout<<"vwgt="<<vwgt<<endl;
@@ -869,25 +885,38 @@ int main(int argc, char *argv[]) {
       scalar_type ratio_size = PARAM.real_value("RATIO_GR_MESH", "ratio size between mesh and patches");
       cout<<"ratio size beween mesh and coarse mesh= "<< ratio_size <<endl;
 
-      int wgtflag = 2, edgecut, nparts=int(size_of_crack/(ratio_size*h)), numflag = 0;
-
+      int nparts = 1;
+#if GETFEM_HAVE_METIS
+      nparts = int(size_of_crack/(ratio_size*h));
+# ifdef GETFEM_HAVE_METIS_OLD_API
+      std::vector<int> adjwgt(k); // actually Metis would also accept NULL instead of an empty array
+      int wgtflag = 2, numflag = 0, edgecut;
+      int options[5] = {0,0,0,0,0};
       // float ubvec[1] = {1.03f};
-      int  options[5] = {0,0,0,0,0};
       //METIS_mCPartGraphKway(&ne, &ncon, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), &(adjwgt[0]), &wgtflag,
       //		    &numflag, &nparts, &(ubvec[0]),  options, &edgecut, &(part[0]));
-      // METIS_mCPartGraphRecursive(&ne, &ncon, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), &(adjwgt[0]), &wgtflag,
+      //METIS_mCPartGraphRecursive(&ne, &ncon, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), &(adjwgt[0]), &wgtflag,
       //			 &numflag, &nparts,  options, &edgecut, &(part[0]));
       //METIS_PartGraphKway(&ne, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), &(adjwgt[0]), &wgtflag,
       //	  &numflag, &nparts, options, &edgecut, &(part[0]));
       METIS_PartGraphRecursive(&ne, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), &(adjwgt[0]), &wgtflag,
-			       &numflag, &nparts, options, &edgecut, &(part[0]));
-      
+                               &numflag, &nparts, options, &edgecut, &(part[0]));
+# else
+      int ncon = 1, edgecut;
+      int options[METIS_NOPTIONS] = { 0 };
+      METIS_SetDefaultOptions(options);
+      METIS_PartGraphRecursive(&ne, &ncon, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), 0, 0,
+                               &nparts, 0, 0, options, &edgecut, &(part[0]));
+# endif
       //cout<<"size_of_mesh="<<h<<endl;
       //cout<<"size_of_crack="<< size_of_crack <<endl;
       cout<<"nb_partition="<<nparts<<endl;
       // cout<<"partition="<<part<<endl;
       //cout<<"edgecut="<<edgecut<<endl;
-      
+#else
+  GMM_ASSERT1(false, "METIS not linked");
+#endif
+
 
       /**************************************************************/
       /*        Assembly matrices                                   */
@@ -1079,8 +1108,6 @@ int main(int argc, char *argv[]) {
   test_mim(mim, mf_rhs, false);
   test_mim(mimbounddown, mf_rhs, true);
 
-#if 1
-
   /***************************************************************/ 
   /*          New brick system                                   */
   /***************************************************************/
@@ -1130,61 +1157,6 @@ int main(int argc, char *argv[]) {
   plain_vector LAMBDA(nb_dof_mult);
   gmm::copy(model.real_variable("Lambda"), LAMBDA);
   
-  // cout<<"desplacement="<<U<<endl;
-  
-  // cout<<"Mult="<<LAMBDA<<endl;
-
-#else
-
-  /***************************************************************/ 
-  /*          Old brick system                                   */
-  /***************************************************************/
-
-  getfem::mdbrick_generic_elliptic<> brick_laplacian(mim, mf);
-
-  getfem::mdbrick_source_term<> brick_volumic_rhs(brick_laplacian);
-  plain_vector F(nb_dof_rhs);
-  getfem::interpolation_function(mf_rhs, F, rhs);
-  brick_volumic_rhs.source_term().set(mf_rhs, F);
-
-  // Neumann condition
-  getfem::interpolation_function(mf_rhs, F, g_exact);
-  plain_vector R(nb_dof);
-  gmm::mult(gmm::transposed(B2), F, R);
-  brick_volumic_rhs.set_auxF(R);
-
-  // Dirichlet condition
-  getfem::mdbrick_constraint<> brick_constraint(brick_volumic_rhs);
-  brick_constraint.set_constraints(B, plain_vector(nb_dof_mult));
-  brick_constraint.set_constraints_type(getfem::AUGMENTED_CONSTRAINTS);
-
-  //add stabilized term
-  if (stabilized_dirichlet > 0){
-    if (stabilized_dirichlet == 3) {
-      brick_constraint.set_optional_matrices(KA, MA);
-    }else{
-      brick_constraint.set_optional_matrices(KA, MA);}
-  }
-
-  getfem::mdbrick_abstract<> *final_brick = &brick_constraint;
-    
-  // Solving the problem
-  cout << "Total number of unknown: " << final_brick->nb_dof() << endl;
-  getfem::standard_model_state MS(*final_brick);
-  gmm::iteration iter(1e-9, 1, 40000);
-  getfem::standard_solve(MS, *final_brick, iter);
-  plain_vector U(nb_dof);
-  gmm::copy(brick_laplacian.get_solution(MS), U);
-  plain_vector LAMBDA(nb_dof_mult);
-  gmm::copy(brick_constraint.get_mult(MS), LAMBDA);
-
- 
-
-#endif
-
-
-
-
   // interpolation of the solution on mf_rhs
   GMM_ASSERT1(!mf_rhs.is_reduced(), "To be adapted");
   plain_vector Uint(nb_dof_rhs), Vint(nb_dof_rhs), Eint(nb_dof_rhs),lambda_int(nb_dof_rhs);
