@@ -1,10 +1,10 @@
 /*===========================================================================
- 
- Copyright (C) 2002-2012 Yves Renard, Julien Pommier.
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2002-2015 Yves Renard, Julien Pommier.
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -16,7 +16,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
 ===========================================================================*/
 
 /**
@@ -25,14 +25,14 @@
  * Research program.
  */
 
-#include "getfem/getfem_assembling.h" /* import assembly methods      */
-#include "getfem/getfem_export.h"     /* export functions             */
+#include "getfem/getfem_assembling.h"
+#include "getfem/getfem_export.h"
 #include "getfem/getfem_derivatives.h"
 #include "getfem/getfem_regular_meshes.h"
 #include "getfem/getfem_model_solvers.h"
 #include "getfem/getfem_mesh_im_level_set.h"
 #include "getfem/getfem_partial_mesh_fem.h"
-#include "getfem/getfem_Coulomb_friction.h"
+#include "getfem/getfem_contact_and_friction_nodal.h"
 #include "getfem/getfem_import.h"
 #include "getfem/getfem_inter_element.h"
 #include "gmm/gmm.h"
@@ -41,7 +41,7 @@ using std::endl; using std::cout; using std::cerr;
 using std::ends; using std::cin;
 
 
-/* some Getfem++ types that we will be using */
+/* some GetFEM++ types that we will be using */
 using bgeot::base_small_vector; /* special class for small (dim<16) vectors */
 using bgeot::base_vector;
 using bgeot::base_node;  /* geometrical nodes(derived from base_small_vector)*/
@@ -191,12 +191,12 @@ public:
   level_set_unit_normal(const getfem::mesh_fem &mf_, const VECT1 &U_) 
     : mf(mf_), U(U_), N(mf_.linked_mesh().dim()), gradU(1, N)
   { sizes_.resize(1); sizes_[0] = short_type(N); }
-  const bgeot::multi_index &sizes() const {  return sizes_; }
+  const bgeot::multi_index &sizes(size_type) const {  return sizes_; }
   virtual void compute(getfem::fem_interpolation_context& ctx,
 		       bgeot::base_tensor &t) {
     size_type cv = ctx.convex_num();
-    coeff.resize(mf.nb_dof_of_element(cv));
-    gmm::copy(gmm::sub_vector(U, gmm::sub_index(mf.ind_dof_of_element(cv))),
+    coeff.resize(mf.nb_basic_dof_of_element(cv));
+    gmm::copy(gmm::sub_vector(U, gmm::sub_index(mf.ind_basic_dof_of_element(cv))),
 	      coeff);
     ctx.pf()->interpolation_grad(ctx, coeff, gradU, 1);
     scalar_type norm = gmm::vect_norm2(gmm::mat_row(gradU, 0));
@@ -301,24 +301,6 @@ void asm_mass_matrix_up(const MAT &RM_, const getfem::mesh_im &mim, const getfem
 
 
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 
 
 
@@ -326,7 +308,8 @@ void asm_mass_matrix_up(const MAT &RM_, const getfem::mesh_im &mim, const getfem
  * Elementary extrapolation matrices
  */
 
-void compute_mass_matrix_extra_element(base_matrix &M, const getfem::mesh_im &mim, const getfem::mesh_fem &mf,
+void compute_mass_matrix_extra_element
+(base_matrix &M, const getfem::mesh_im &mim, const getfem::mesh_fem &mf,
  size_type cv1, size_type cv2) {
 
   getfem::pfem pf1_old = 0;
@@ -355,7 +338,7 @@ void compute_mass_matrix_extra_element(base_matrix &M, const getfem::mesh_im &mi
   }
   
   bgeot::vectors_to_base_matrix(G1, m.points_of_convex(cv1));
-  getfem::fem_interpolation_context ctx1(pgt1, pfp1, 0, G1, cv1,size_type(-1));
+  getfem::fem_interpolation_context ctx1(pgt1, pfp1, 0, G1, cv1, short_type(-1));
   
   getfem::pfem pf2 = mf.fem_of_element(cv2);
   size_type nbd2 = pf1->nb_dof(cv2);
@@ -368,7 +351,7 @@ void compute_mass_matrix_extra_element(base_matrix &M, const getfem::mesh_im &mi
   bgeot::vectors_to_base_matrix(G2, m.points_of_convex(cv2));
   
   getfem::fem_interpolation_context ctx2(pgt2, pf2, base_node(pgt2->dim()),
-					 G2, cv2, size_type(-1));
+					 G2, cv2, short_type(-1));
 
   for (unsigned ii=0; ii < pai1->nb_points_on_convex(); ++ii) {
     ctx1.set_ii(ii);
@@ -453,7 +436,7 @@ int main(int argc, char *argv[]) {
   //getfem::level_set lsup(mesh, dim_type(lsdeg), true), lsdown(mesh, dim_type(lsdeg), true);
   const getfem::mesh_fem &lsmf = ls.get_mesh_fem();
   for (unsigned i = 0; i < lsmf.nb_dof(); ++i) {
-    ls.values()[i] = ls_value(lsmf.point_of_dof(i));
+    ls.values()[i] = ls_value(lsmf.point_of_basic_dof(i));
 	  }
  
 	
@@ -522,7 +505,7 @@ int main(int argc, char *argv[]) {
   // Finite element method for the unknown velocity u
   getfem::mesh_fem pre_mf(mesh);
   std::string FEM = PARAM.string_value("FEM", "finite element method");
-  pre_mf.set_qdim(N);
+  pre_mf.set_qdim(bgeot::dim_type(N));
   pre_mf.set_finite_element(mesh.convex_index(),
 			    getfem::fem_descriptor(FEM));
 	
@@ -556,7 +539,7 @@ int main(int argc, char *argv[]) {
   // Finite element method for the rhs
   getfem::mesh_fem mf_rhs(mesh);
   std::string FEMR = PARAM.string_value("FEM_RHS", "finite element method");
-  mf_rhs.set_qdim(N);
+  mf_rhs.set_qdim(bgeot::dim_type(N));
   mf_rhs.set_finite_element(mesh.convex_index(),
 			    getfem::fem_descriptor(FEMR));
   size_type nb_dof_rhs = mf_rhs.nb_dof();
@@ -568,7 +551,7 @@ int main(int argc, char *argv[]) {
   // Finite element method for the multipliers
   getfem::mesh_fem pre_mf_mult(mesh);
   std::string FEMM = PARAM.string_value("FEM_MULT", "fem for multipliers");
-  pre_mf_mult.set_qdim(N);
+  pre_mf_mult.set_qdim(bgeot::dim_type(N));
   pre_mf_mult.set_finite_element(mesh.convex_index(),
 				 getfem::fem_descriptor(FEMM));
   getfem::partial_mesh_fem mf_mult(pre_mf_mult);
@@ -673,7 +656,7 @@ int main(int argc, char *argv[]) {
       getfem::asm_mass_matrix(MC1, mim, mf_P0);
       getfem::asm_mass_matrix(MC2, uncutmim, mf_P0);
       for (size_type i = 0; i < nbe; ++i) {
-	size_type cv = mf_P0.first_convex_of_dof(i);
+	size_type cv = mf_P0.first_convex_of_basic_dof(i);
 	ratios[cv] = gmm::abs(MC1(i,i)) / gmm::abs(MC2(i,i));
 	if (ratios[cv] > 0 && ratios[cv] < min_ratio) elt_black_list.add(cv);
       }
@@ -685,7 +668,7 @@ int main(int argc, char *argv[]) {
 
       for (size_type i = 0; i < nb_dof; ++i) {
 	bool found = false;
-	getfem::mesh::ind_cv_ct ct = mf.convex_to_dof(i);
+	getfem::mesh::ind_cv_ct ct = mf.convex_to_basic_dof(i);
 	getfem::mesh::ind_cv_ct::const_iterator it;
 	for (it = ct.begin(); it != ct.end(); ++it)
 	  if (!elt_black_list.is_in(*it)) found = true;
@@ -709,7 +692,7 @@ int main(int argc, char *argv[]) {
 	compute_mass_matrix_extra_element(Mloc, uncutmim, mf, i, cv2);
 	for (size_type ii = 0; ii < gmm::mat_nrows(Mloc); ++ii) 
 	  for (size_type jj = 0; jj < gmm::mat_ncols(Mloc); ++jj)
-	    EX(mf.ind_dof_of_element(i)[ii], mf.ind_dof_of_element(cv2)[jj])
+	    EX(mf.ind_basic_dof_of_element(i)[ii], mf.ind_basic_dof_of_element(cv2)[jj])
 	      += Mloc(ii, jj);
       }
 
@@ -928,7 +911,7 @@ cout << compteur << " " << G[0] << " " << G[1] << endl;
   // computation of error on u.
   double errmax = 0.0, exactmax = 0.0;
   for (size_type i = 0; i < nb_dof_rhs; ++i)
-    if (ls_value(mf_rhs.point_of_dof(i)) >= 0.0) {
+    if (ls_value(mf_rhs.point_of_basic_dof(i)) >= 0.0) {
       errmax = std::max(errmax, Eint[i]);
       exactmax = std::max(exactmax, Vint[i]);
     }
