@@ -1,10 +1,10 @@
 /*===========================================================================
- 
- Copyright (C) 2006-2012 Yves Renard, Julien Pommier.
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2006-2015 Yves Renard, Julien Pommier.
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -16,7 +16,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
 ===========================================================================*/
 
 #include <getfemint_misc.h>
@@ -145,7 +145,7 @@ gf_interpolate(getfemint::mexargs_in& in, getfemint::mexargs_out& out,
     sl->interpolate(mf, U, V);
   }
   else {
-    size_type N = mf.get_qdim();
+    size_type N = mf.linked_mesh().dim();
     darray st = in.pop().to_darray();
     std::vector<double> PTS(st.begin(), st.end());
     size_type nbpoints = gmm::vect_size(PTS) / N;
@@ -153,6 +153,7 @@ gf_interpolate(getfemint::mexargs_in& in, getfemint::mexargs_out& out,
     getfem::mesh_trans_inv mti(mf.linked_mesh());
     for (size_type i = 0; i < nbpoints; ++i) {
       gmm::copy(gmm::sub_vector(PTS, gmm::sub_interval(i*N, N)), p);
+      // cout << "adding point" << p << endl;
       mti.add_point(p);
     }
     
@@ -163,7 +164,9 @@ gf_interpolate(getfemint::mexargs_in& in, getfemint::mexargs_out& out,
     garray<T> V = out.pop().create_array(dims,T());
 
     getfem::base_matrix Maux;
+    // cout << "begin interpolation, qmult = " << qmult << endl;
     getfem::interpolation(mf, mti, U, V, Maux, 0);
+    // cout << "end interpolation" << endl;
 
   }
   // else THROW_BADARG("expecting a mesh_fem or a mesh_slice for interpolation");
@@ -514,19 +517,58 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
 	 getfem::error_estimate(mim, *mf, U.cplx(), err, mim.convex_index());
        );
 
-    /*@FUNC E = ('convect', @tmf mf_v, @dvec V, @scalar dt, @int nt[, @str option])
+      
+#ifdef EXPERIMENTAL_PURPOSE_ONLY
+            
+    /*@FUNC E = ('error estimate nitsche', @tmim mim, @int GAMMAC, @int GAMMAN, @scalar lambda_, @scalar mu_, @scalar gamma0, @scalar f_coeff, @scalar vertical_force)
+    Compute an a posteriori error estimate in the case of Nitsche method.
+
+    Currently there is only one which is available: for each convex,
+    the jump of the normal derivative is integrated on its faces.@*/
+    sub_command
+      ("error_estimate_nitsche", 8, 8, 0, 1,
+       const getfem::mesh_im &mim = *in.pop().to_const_mesh_im();
+       int GAMMAC = in.pop().to_integer();
+       int GAMMAN = in.pop().to_integer();
+       scalar_type lambda = in.pop().to_scalar();
+       scalar_type mu = in.pop().to_scalar();
+       scalar_type gamma0 = in.pop().to_scalar();
+       scalar_type f_coeff = in.pop().to_scalar();
+       scalar_type vertical_force = in.pop().to_scalar();
+       unsigned si = unsigned(mim.linked_mesh().convex_index().last_true()+1);
+       darray err =
+       out.pop().create_darray_h(si);
+       getfem::base_vector ERR(si);
+       getfem::base_vector UU(U.real().size());
+       gmm::copy(U.real(), UU);
+       getfem::error_estimate_nitsche(mim, *mf, UU, GAMMAC, GAMMAN, lambda, mu, gamma0, f_coeff,vertical_force, ERR);
+       gmm::copy(ERR, err);
+       );   
+      
+#endif
+      
+      
+      
+      
+    /*@FUNC ('convect', @tmf mf_v, @dvec V, @scalar dt, @int nt[, @str option[, @dvec per_min, @dvec per_max]])
     Compute a convection of `U` with regards to a steady state velocity
-    field `V` with a Characteristic-Galerkin method. This
-    method is restricted to pure Lagrange fems for U. `mf_v` should represent
-    a continuous finite element method. `dt` is the integration time and `nt`
-    is the number of integration step on the caracteristics. `option` is an
-    option for the part of the boundary where there is a re-entrant convection.
-    `option = 'extrapolation'` for an extrapolation on the nearest element
-    or `option = 'unchanged'` for a constant value on that boundary.
+    field `V` with a Characteristic-Galerkin method. The result is returned
+    in-place in `U`.
+    This method is restricted to pure Lagrange fems for U. `mf_v` should
+    represent a continuous finite element method. `dt` is the integration time
+    and `nt` is the number of integration step on the caracteristics. `option`
+    is an option for the part of the boundary where there is a re-entrant
+    convection.
+    `option = 'extrapolation'` for an extrapolation on the nearest element,
+    `option = 'unchanged'` for a constant value on that boundary or
+    `option = 'periodicity'` for a peridiodic boundary. For this latter option
+    the two vectors per_min, per_max has to be given and represent the limits
+    of the periodic domain (on components where per_max[k] < per_min[k]
+    no operation is done).
     This method is rather dissipative, but stable.
     @*/
     sub_command
-      ("convect", 4, 5, 0, 0,
+      ("convect", 4, 7, 0, 0,
        const getfem::mesh_fem *mf_v = in.pop().to_const_mesh_fem();
        rcarray V              = in.pop().to_rcarray();
        scalar_type dt = in.pop().to_scalar();
@@ -538,15 +580,31 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
 	 opt = getfem::CONVECT_EXTRAPOLATION;
        else if (cmd_strmatch(option, "extrapolation"))
 	 opt = getfem::CONVECT_EXTRAPOLATION;
+       else if (cmd_strmatch(option, "periodicity"))
+	 opt = getfem::CONVECT_PERIODICITY;
        else if (cmd_strmatch(option, "unchanged"))
 	 opt = getfem::CONVECT_UNCHANGED;
        else 
 	 THROW_BADARG("Bad option " << option<< " for convect command. "
-		      "should be 'extrapolation' or 'unchanged'");
+		      "should be 'extrapolation', 'unchanged' or "
+                      "'periodicity'");
+
+       getfem::base_node per_min;
+       getfem::base_node per_max;
+       if (in.remaining()) {
+         rcarray pmin = in.pop().to_rcarray();
+         rcarray pmax = in.pop().to_rcarray();
+         size_type N = mf_v->linked_mesh().dim();
+         per_min.resize(N);
+         per_max.resize(N); 
+         gmm::copy(pmin.real(), per_min);
+         gmm::copy(pmax.real(), per_max);
+       }
 
        if (U.is_complex() || V.is_complex())
 	 THROW_BADARG("Sorry, complex version of convect to be interfaced");
-       getfem::convect(*mf, U.real(), *mf_v, V.real(), dt, nt, opt);
+       getfem::convect(*mf, U.real(), *mf_v, V.real(),
+                       dt, nt, opt, per_min, per_max);
 
        );
 
