@@ -1,10 +1,10 @@
 /*===========================================================================
- 
- Copyright (C) 2009-2012 Yves Renard.
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2009-2015 Yves Renard.
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -16,7 +16,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
 ===========================================================================*/
 
 /**\file gf_model_get.cc
@@ -28,24 +28,27 @@
 #include <getfemint_misc.h>
 #include <getfemint_models.h>
 #include <getfem/getfem_model_solvers.h>
-#include <getfemint_mdbrick.h>
+#include <getfem/getfem_generic_assembly.h>
+#include <getfem/getfem_nonlinear_elasticity.h>
+#include <getfem/getfem_plasticity.h>
+#include <getfem/getfem_contact_and_friction_large_sliding.h>
 #include <getfemint_mesh_fem.h>
 #include <getfemint_mesh_im.h>
 
 using namespace getfemint;
 
 
-#define RETURN_SPARSE(realmeth, cplxmeth)                              \
-  if (!md->is_complex()) {                                             \
+#define RETURN_SPARSE(realmeth, cplxmeth)                            \
+  if (!md->is_complex()) {                                           \
     gf_real_sparse_by_col M(gmm::mat_nrows(md->model().realmeth),    \
                             gmm::mat_ncols(md->model().realmeth));   \
     gmm::copy(md->model().realmeth, M);                              \
-    out.pop().from_sparse(M);                                          \
-  } else {                                                             \
+    out.pop().from_sparse(M);                                        \
+  } else {                                                           \
     gf_cplx_sparse_by_col M(gmm::mat_nrows(md->model().cplxmeth),    \
                             gmm::mat_ncols(md->model().cplxmeth));   \
     gmm::copy(md->model().cplxmeth, M);                              \
-    out.pop().from_sparse(M);                                          \
+    out.pop().from_sparse(M);                                        \
   }
 
 #define RETURN_VECTOR(realmeth, cplxmeth)                      \
@@ -110,6 +113,21 @@ void gf_model_get(getfemint::mexargs_in& m_in,
        out.pop().from_integer(int(md->model().nb_dof()));
        );
 
+    /*@GET dt = ('get time step')
+      Gives the value of the time step. @*/
+    sub_command
+      ("get time step", 0, 0, 0, 1,
+       out.pop().from_scalar(md->model().get_time_step());
+       );
+
+    /*@GET t = ('get time')
+      Give the value of the data `t` corresponding to the current time.
+      @*/
+    sub_command
+      ("get time", 0, 0, 0, 1,
+       out.pop().from_scalar(md->model().get_time());
+       );
+
     /*@GET T = ('tangent_matrix')
       Return the tangent matrix stored in the model .@*/
     sub_command
@@ -117,7 +135,6 @@ void gf_model_get(getfemint::mexargs_in& m_in,
        RETURN_SPARSE(real_tangent_matrix(), complex_tangent_matrix());
        );
 
-
     /*@GET ('rhs')
       Return the right hand side of the tangent problem.@*/
     sub_command
@@ -167,34 +184,87 @@ void gf_model_get(getfemint::mexargs_in& m_in,
        );
 
 
-    /*@GET ('listvar')
+    /*@GET ('variable list')
       print to the output the list of variables and constants of the model.@*/
     sub_command
-      ("listvar", 0, 0, 0, 0,
+      ("variable list", 0, 0, 0, 0,
        md->model().listvar(infomsg());
        );
 
 
-    /*@GET ('listbricks')
+    /*@GET ('brick list')
       print to the output the list of bricks of the model.@*/
     sub_command
-      ("listbricks", 0, 0, 0, 0,
+      ("brick list", 0, 0, 0, 0,
        md->model().listbricks(infomsg(), config::base_index());
        );
 
-
-    /*@GET V = ('variable', @str name[, @int niter])
-      Gives the value of a variable or data.@*/
+    /*@GET ('list residuals')
+      print to the output the residuals corresponding to all terms
+      included in the model.@*/
     sub_command
-      ("variable", 1, 2, 0, 1,
-       std::string name = in.pop().to_string();
-       size_type niter = 0;
-       if (in.remaining())
-         niter = in.pop().to_integer(0,10) - config::base_index();
-       RETURN_VECTOR(real_variable(name, niter),
-                     complex_variable(name, niter));
+      ("list residuals", 0, 0, 0, 0,
+       md->model().listresiduals(cout);
        );
 
+
+    /*@GET V = ('variable', @str name)
+      Gives the value of a variable or data. @*/
+    sub_command
+      ("variable", 1, 1, 0, 1,
+       std::string name = in.pop().to_string();
+       RETURN_VECTOR(real_variable(name), complex_variable(name));
+       );
+
+
+    /*@GET V = ('interpolation', @str expr, {@tmf mf | @tmimd mimd | @vec pts,  @tmesh m}[, @int region[, @int extrapolation[, @int rg_source]]])
+      Interpolate a certain expression with respect to the mesh_fem `mf`
+      or the mesh_im_data `mimd` or the set of points `pts` on mesh `m`.
+      The expression has to be valid according to the high-level generic
+      assembly language possibly including references to the variables
+      and data of the model.
+
+      The options `extrapolation` and `rg_source` are specific to
+      interpolations with respect to a set of points `pts`. @*/
+    sub_command
+      ("interpolation", 2, 6, 0, 1, // should be extended to complex models ...
+       std::string expr = in.pop().to_string();
+       getfem::base_vector result;
+       if (in.front().is_mesh_fem()) {
+         const getfem::mesh_fem *mf = in.pop().to_const_mesh_fem();
+         size_type rg = in.remaining() ? in.pop().to_integer()
+                                       : size_type(-1);
+         getfem::ga_interpolation_Lagrange_fem(md->model(), expr,
+                                               *mf, result, rg);
+       } else if (in.front().is_mesh_im_data()) {
+         getfem::im_data *mimd = in.pop().to_mesh_im_data();
+         size_type rg = in.remaining() ? in.pop().to_integer()
+                                       : size_type(-1);
+         getfem::ga_interpolation_im_data(md->model(), expr,
+                                          *mimd, result, rg);
+       } else {
+         darray st = in.pop().to_darray();
+         std::vector<double> PTS(st.begin(), st.end());
+         const getfem::mesh *m = in.pop().to_const_mesh();
+         size_type N = m->dim();
+         size_type nbpoints = gmm::vect_size(PTS) / N;
+         getfem::base_node p(N);
+         getfem::mesh_trans_inv mti(*m);
+         for (size_type i = 0; i < nbpoints; ++i) {
+           gmm::copy(gmm::sub_vector(PTS, gmm::sub_interval(i*N, N)), p);
+           mti.add_point(p);
+         }
+         size_type rg = in.remaining() ? in.pop().to_integer()
+                                       : size_type(-1);
+         int extrapolation = in.remaining() ? in.pop().to_integer()
+                                            : size_type(0);
+         size_type rg_source = in.remaining() ? in.pop().to_integer()
+                                              : size_type(-1);
+         getfem::ga_interpolation_mti(md->model(), expr, mti,
+                                      result, rg, extrapolation, rg_source);
+       }
+       out.pop().from_dcvector(result);
+       );
     
     /*@GET mf = ('mesh fem of variable', @str name)
       Gives access to the `mesh_fem` of a variable or data.@*/
@@ -204,9 +274,12 @@ void gf_model_get(getfemint::mexargs_in& m_in,
        const getfem::mesh_fem &mf = md->model().mesh_fem_of_variable(name);
        getfem::mesh_fem *mmf = const_cast<getfem::mesh_fem *>(&mf);
        getfem_object *o =
-       getfemint::workspace().object(getfem_object::internal_key_type(mmf));
+         getfemint::workspace().object(getfem_object::internal_key_type(mmf));
        getfemint_mesh_fem *gmf = getfemint_mesh_fem::get_from(mmf);
-       if (!o) workspace().set_dependance(gmf, md);
+       if (!o) {
+         gmf->set_flags(STATIC_OBJ);
+         workspace().set_dependance(gmf, md);
+       }
        out.pop().from_object_id(gmf->get_id(), MESHFEM_CLASS_ID);
        );
 
@@ -257,13 +330,12 @@ void gf_model_get(getfemint::mexargs_in& m_in,
     /*@GET ('assembly'[, @str option])
       Assembly of the tangent system taking into account the terms
       from all bricks. `option`, if specified, should be 'build_all',
-      'build_rhs', 'build_matrix' or 'pseudo_potential' (in that case,
-      the pseudo_potential is returned).
+      'build_rhs', 'build_matrix'.
       The default is to build the whole
       tangent linear system (matrix and rhs). This function is useful
       to solve your problem with you own solver. @*/
     sub_command
-      ("assembly", 0, 1, 0, 1,
+      ("assembly", 0, 1, 0, 0,
        std::string option = "build_all";
        if (in.remaining()) option = in.pop().to_string();
        getfem::model::build_version version = getfem::model::BUILD_ALL;
@@ -276,13 +348,8 @@ void gf_model_get(getfemint::mexargs_in& m_in,
        else if (cmd_strmatch(option, "build matrix") ||
                 cmd_strmatch(option, "build_matrix"))
          version = getfem::model::BUILD_MATRIX;
-       else if (cmd_strmatch(option, "pseudo potential") ||
-                cmd_strmatch(option, "pseudo_potential"))
-         version = getfem::model::BUILD_PSEUDO_POTENTIAL;
        else THROW_BADARG("bad option: " << option);
        md->model().assembly(version);
-       if (version == getfem::model::BUILD_PSEUDO_POTENTIAL)
-         out.pop().from_scalar(md->model().pseudo_potential());
        );
 
 
@@ -314,36 +381,30 @@ void gf_model_get(getfemint::mexargs_in& m_in,
        select explicitely the line search method used for the linear systems (the
        default value is 'default').
        Possible values are 'simplest', 'systematic', 'quadratic' or 'basic'.
-    - 'with pseudo potential'
-      for nonlinear problems, the criterion of the line search will
-      be a pseudo potential instead of the residual. Still experimental since
-      not all bricks define a pseudo potential.
 
-      Return the number of iterations, if a iterative method is used.
+      Return the number of iterations, if an iterative method is used.
       
       Note that it is possible to disable some variables
       (see MODEL:SET('disable variable') ) in order to
       solve the problem only with respect to a subset of variables (the
-      disabled variables are the considered as data) for instance to
+      disabled variables are then considered as data) for instance to
       replace the global Newton strategy with a fixed point one.
 
       @*/
     sub_command
-      ("solve", 0, 15, 0, 2,
+      ("solve", 0, 17, 0, 2,
        getfemint::interruptible_iteration iter;
        std::string lsolver = "auto";
        std::string lsearch = "default";
-       bool with_pseudo_pot = false;
-       scalar_type alpha_mult = 0.5;
-       scalar_type alpha_min = 1.0/1000.0;
-       scalar_type alpha_max_ratio = 6.0/5.0;
+       scalar_type alpha_max_ratio(-1);
+       scalar_type alpha_min(-1);
+       scalar_type alpha_mult(-1);
+       scalar_type alpha_threshold_res(1e50);
        while (in.remaining() && in.front().is_string()) {
          std::string opt = in.pop().to_string();
          if (cmd_strmatch(opt, "noisy")) iter.set_noisy(1);
-         else if (cmd_strmatch(opt, "with pseudo potential"))
-           with_pseudo_pot = true;
          else if (cmd_strmatch(opt, "very noisy") ||
-                  cmd_strmatch(opt, "very_noisy")) iter.set_noisy(2);
+                  cmd_strmatch(opt, "very_noisy")) iter.set_noisy(3);
          else if (cmd_strmatch(opt, "max_iter")) {
            if (in.remaining()) iter.set_maxiter(in.pop().to_integer());
            else THROW_BADARG("missing value for " << opt);
@@ -361,20 +422,32 @@ void gf_model_get(getfemint::mexargs_in& m_in,
            else THROW_BADARG("missing line search name for " << opt);
          } else if (cmd_strmatch(opt, "alpha mult")) {
            if (in.remaining()) alpha_mult = in.pop().to_scalar();
-           else THROW_BADARG("missing line search name for " << opt);
+           else THROW_BADARG("missing line search value for " << opt);
          } else if (cmd_strmatch(opt, "alpha min")) {
            if (in.remaining()) alpha_min = in.pop().to_scalar();
-           else THROW_BADARG("missing line search name for " << opt);
+           else THROW_BADARG("missing line search value for " << opt);
          } else if (cmd_strmatch(opt, "alpha max ratio")) {
            if (in.remaining()) alpha_max_ratio = in.pop().to_scalar();
-           else THROW_BADARG("missing line search name for " << opt);
+           else THROW_BADARG("missing line search value for " << opt);
+         } else if (cmd_strmatch(opt, "alpha threshold res")) {
+           if (in.remaining()) alpha_threshold_res = in.pop().to_scalar();
+           else THROW_BADARG("missing line search value for " << opt);
          } else THROW_BADARG("bad option: " << opt);
        }
 
+       // default values in sync with getfem_model_solvers.h
+       if (alpha_max_ratio < scalar_type(0))
+         alpha_max_ratio = (lsearch == "basic") ?  5.0/3.0 : 6.0/5.0;
+       if (alpha_min < scalar_type(0))
+         alpha_min = (lsearch == "systematic") ? 1.0/10000.0 : 1.0/1000.0;
+       if (alpha_mult < scalar_type(0))
+         alpha_mult = 3.0/5.0;
+
        getfem::default_newton_line_search default_ls;
-       getfem::simplest_newton_line_search simplest_ls(size_type(-1), alpha_max_ratio, alpha_min, alpha_mult);
+       getfem::simplest_newton_line_search simplest_ls(size_type(-1), alpha_max_ratio,
+                                                       alpha_min, alpha_mult, alpha_threshold_res);
        getfem::systematic_newton_line_search systematic_ls(size_type(-1), alpha_min, alpha_mult);
-       getfem::basic_newton_line_search basic_ls(size_type(-1), alpha_min, alpha_mult);
+       getfem::basic_newton_line_search basic_ls(size_type(-1), alpha_max_ratio, alpha_min, alpha_mult);
        getfem::quadratic_newton_line_search quadratic_ls(size_type(-1));
 
        getfem::abstract_newton_line_search *ls = 0;
@@ -395,13 +468,11 @@ void gf_model_get(getfemint::mexargs_in& m_in,
        if (!md->model().is_complex()) {
          getfem::standard_solve(md->model(), iter,
                                 getfem::rselect_linear_solver(md->model(),
-                                                              lsolver),
-                                *ls, with_pseudo_pot);
+                                                              lsolver), *ls);
        } else {
          getfem::standard_solve(md->model(), iter,
                                 getfem::cselect_linear_solver(md->model(),
-                                                              lsolver),
-                                *ls, with_pseudo_pot);
+                                                              lsolver), *ls);
        }
        if (out.remaining()) out.pop().from_integer(int(iter.get_iteration()));
        if (out.remaining()) out.pop().from_integer(int(iter.converged()));
@@ -414,8 +485,10 @@ void gf_model_get(getfemint::mexargs_in& m_in,
       `EPS` is the value of the small parameter for the finite difference
       computation of the derivative is the random direction (default is 1E-6).
       `NN` is the number of tests (default is 100). `scale` is a parameter
-      for the random position (default is 1). Each dof od the random
-      position is chosen in the range [-scale, scale].
+      for the random position (default is 1, 0 is an acceptable value) around
+      the current position.
+      Each dof of the random position is chosen in the range
+      [current-scale, current+scale].
       @*/
     sub_command
       ("test tangent matrix", 0, 3, 0, 1,
@@ -425,26 +498,29 @@ void gf_model_get(getfemint::mexargs_in& m_in,
        scalar_type errmax = scalar_type(0);
        size_type NN = 100;
        if (in.remaining()) NN = in.pop().to_integer();
-       scalar_type scale = scalar_type(1);
-       if (in.remaining()) scale = in.pop().to_scalar();
+       scalar_type scalef = scalar_type(1);
+       if (in.remaining()) scalef = in.pop().to_scalar();
 
-       if (md->model().is_linear())
-	 cout << "Problem is linear, the test is not relevant";
-       else {
+       if (md->model().is_linear()) {
+	 GMM_WARNING1("Problem is linear, the test is not relevant");
+       } else {
 	 if (md->is_complex()) {
 	   std::vector<complex_type> U(nbdof);
+	   std::vector<complex_type> dU(nbdof);
 	   std::vector<complex_type> DIR(nbdof);
 	   std::vector<complex_type> D1(nbdof);
 	   std::vector<complex_type> D2(nbdof);
+           md->model().from_variables(U);
 	   for (size_type i = 0; i < NN; ++i) {
-	     gmm::fill_random(U); gmm::scale(U, scale);
-	     gmm::fill_random(DIR); gmm::scale(DIR, scale);
-	     md->model().to_variables(U);
+	     gmm::fill_random(dU); gmm::scale(dU, complex_type(scalef));
+             gmm::add(U, dU);
+	     gmm::fill_random(DIR);
+	     md->model().to_variables(dU);
 	     md->model().assembly(getfem::model::BUILD_ALL);
 	     gmm::copy(md->model().complex_rhs(), D2);
 	     gmm::mult(md->model().complex_tangent_matrix(), DIR, D1);
-	     gmm::add(gmm::scaled(DIR, complex_type(EPS)), U);
-	     md->model().to_variables(U);
+	     gmm::add(gmm::scaled(DIR, complex_type(EPS)), dU);
+	     md->model().to_variables(dU);
 	     md->model().assembly(getfem::model::BUILD_RHS);
 	     gmm::add(gmm::scaled(md->model().complex_rhs(),
 				  -complex_type(1)), D2);
@@ -453,20 +529,24 @@ void gf_model_get(getfemint::mexargs_in& m_in,
 	     cout << "Error at step " << i << " : " << err << endl;
 	     errmax = std::max(err, errmax);
 	   }
+           md->model().to_variables(U);
 	 } else {
 	   std::vector<scalar_type> U(nbdof);
+	   std::vector<scalar_type> dU(nbdof);
 	   std::vector<scalar_type> DIR(nbdof);
 	   std::vector<scalar_type> D1(nbdof);
 	   std::vector<scalar_type> D2(nbdof);
+           md->model().from_variables(U);
 	   for (size_type i = 0; i < NN; ++i) {
-	     gmm::fill_random(U); gmm::scale(U, scale);
-	     gmm::fill_random(DIR); gmm::scale(DIR, scale);
-	     md->model().to_variables(U);
+	     gmm::fill_random(dU); gmm::scale(dU, scalef);
+             gmm::add(U, dU);
+	     gmm::fill_random(DIR);
+	     md->model().to_variables(dU);
 	     md->model().assembly(getfem::model::BUILD_ALL);
 	     gmm::copy(md->model().real_rhs(), D2);
 	     gmm::mult(md->model().real_tangent_matrix(), DIR, D1);
-	     gmm::add(gmm::scaled(DIR, EPS), U);
-	     md->model().to_variables(U);
+	     gmm::add(gmm::scaled(DIR, EPS), dU);
+	     md->model().to_variables(dU);
 	     md->model().assembly(getfem::model::BUILD_RHS);
 	     gmm::add(gmm::scaled(md->model().real_rhs(),-scalar_type(1)), D2);
 	     gmm::scale(D2, scalar_type(1)/EPS);
@@ -474,16 +554,131 @@ void gf_model_get(getfemint::mexargs_in& m_in,
 	     cout << "Error at step " << i << " : " << err << endl;
 	     errmax = std::max(err, errmax);
 	   }
+           md->model().to_variables(U);
 	 }
        }
        out.pop().from_scalar(errmax);
        );
 
 
+    /*@GET ('test tangent matrix term', @str varname1, @str varname2[, @scalar EPS[, @int NB[, @scalar scale]]])
+      Test the consistency of a part of the tangent matrix in some
+      random positions and random directions
+      (useful to test newly created bricks).
+      The increment is only made on variable `varname2` and tested on the
+      part of the residual corresponding to `varname1`. This means that
+      only the term (`varname1`, `varname2`) of the tangent matrix is tested.
+      `EPS` is the value of the small parameter for the finite difference
+      computation of the derivative is the random direction (default is 1E-6).
+      `NN` is the number of tests (default is 100). `scale` is a parameter
+      for the random position (default is 1, 0 is an acceptable value)
+      around the current position.
+      Each dof of the random position is chosen in the range
+      [current-scale, current+scale].
+      @*/
+    sub_command
+      ("test tangent matrix term", 2, 5, 0, 1,
+       std::string varname1 = in.pop().to_string();
+       std::string varname2 = in.pop().to_string();
+       gmm::sub_interval I1 = md->model().interval_of_variable(varname1);
+       gmm::sub_interval I2 = md->model().interval_of_variable(varname2);
+       size_type nbdof1 = I1.size();
+       size_type nbdof2 = I2.size();
+
+       scalar_type EPS = 1E-6;
+       if (in.remaining()) EPS = in.pop().to_scalar();
+       scalar_type errmax = scalar_type(0);
+       size_type NN = 100;
+       if (in.remaining()) NN = in.pop().to_integer();
+       scalar_type scalef = scalar_type(1);
+       if (in.remaining()) scalef = in.pop().to_scalar();
+
+       if (md->model().is_linear()) {
+	 GMM_WARNING1("Problem is linear, the test is not relevant");
+       } else {
+	 if (md->is_complex()) {
+	   std::vector<complex_type> U2(nbdof2);
+	   std::vector<complex_type> dU2(nbdof2);
+	   std::vector<complex_type> DIR2(nbdof2);
+	   std::vector<complex_type> D1(nbdof1);
+	   std::vector<complex_type> D2(nbdof1);
+           gmm::copy(md->model().complex_variable(varname2), U2);
+	   for (size_type i = 0; i < NN; ++i) {
+	     gmm::fill_random(dU2); gmm::scale(dU2, complex_type(scalef));
+             gmm::add(U2, dU2);
+             gmm::copy(dU2, md->model().set_complex_variable(varname2));
+	     gmm::fill_random(DIR2);
+	     md->model().assembly(getfem::model::BUILD_ALL);
+	     gmm::copy(gmm::sub_vector(md->model().complex_rhs(), I1), D2);
+	     gmm::mult(gmm::sub_matrix(md->model().complex_tangent_matrix(),
+                                       I1, I2), DIR2, D1);
+	     gmm::add(gmm::scaled(DIR2, complex_type(EPS)), dU2);
+	     gmm::copy(dU2, md->model().set_complex_variable(varname2));
+	     md->model().assembly(getfem::model::BUILD_RHS);
+	     gmm::add(gmm::scaled(gmm::sub_vector(md->model().complex_rhs(),
+                                                  I1), complex_type(-1)), D2);
+	     gmm::scale(D2, complex_type(1)/complex_type(EPS));
+	     scalar_type err = gmm::vect_dist2(D1, D2);
+	     cout << "Error at step " << i << " : " << err << endl;
+	     errmax = std::max(err, errmax);
+	   }
+           gmm::copy(U2, md->model().set_complex_variable(varname2));
+	 } else {
+	   std::vector<scalar_type> U2(nbdof2);
+	   std::vector<scalar_type> dU2(nbdof2);
+	   std::vector<scalar_type> DIR2(nbdof2);
+	   std::vector<scalar_type> D1(nbdof1);
+	   std::vector<scalar_type> D2(nbdof1);
+           gmm::copy(md->model().real_variable(varname2), U2);
+	   for (size_type i = 0; i < NN; ++i) {
+	     gmm::fill_random(dU2); gmm::scale(dU2, scalef);
+             gmm::add(U2, dU2);
+             gmm::copy(dU2, md->model().set_real_variable(varname2));
+	     gmm::fill_random(DIR2);
+	     md->model().assembly(getfem::model::BUILD_ALL);
+	     gmm::copy(gmm::sub_vector(md->model().real_rhs(), I1), D2);
+	     gmm::mult(gmm::sub_matrix(md->model().real_tangent_matrix(),
+                                       I1, I2), DIR2, D1);
+	     gmm::add(gmm::scaled(DIR2, scalar_type(EPS)), dU2);
+	     gmm::copy(dU2, md->model().set_real_variable(varname2));
+	     md->model().assembly(getfem::model::BUILD_RHS);
+	     gmm::add(gmm::scaled(gmm::sub_vector(md->model().real_rhs(),
+                                                  I1), scalar_type(-1)), D2);
+	     gmm::scale(D2, scalar_type(1)/EPS);
+	     scalar_type err = gmm::vect_dist2(D1, D2);
+	     cout << "Error at step " << i << " : " << err << endl;
+	     errmax = std::max(err, errmax);
+	   }
+           gmm::copy(U2, md->model().set_real_variable(varname2));
+	 }
+       }
+       out.pop().from_scalar(errmax);
+       );
+
+    /*@GET expr = ('Neumann term', @str varname, @int region)
+       Gives the assembly string corresponding to the Neumann term of
+       the fem variable `varname` on `region`. It is deduced from the
+       assembly string declared by the model bricks.
+       `region` should be the index of a boundary region
+       on the mesh where `varname` is defined. Care to call this function
+       only after all the volumic bricks have been declared.
+       Complains, if a brick
+       omit to declare an assembly string. @*/
+    sub_command
+      ("Neumann term", 2, 2, 0, 1,
+       std::string varname = in.pop().to_string();
+       size_type rg =in.pop().to_integer();
+       std::string expr = md->model().Neumann_term(varname, rg);
+       out.pop().from_string(expr.c_str());
+       );
+
+
     /*@GET V = ('compute isotropic linearized Von Mises or Tresca', @str varname, @str dataname_lambda, @str dataname_mu, @tmf mf_vm[, @str version])
       Compute the Von-Mises stress or the Tresca stress of a field (only
       valid for isotropic linearized elasticity in 3D). `version` should
-      be  'Von_Mises' or 'Tresca' ('Von_Mises' is the default). @*/
+      be  'Von_Mises' or 'Tresca' ('Von_Mises' is the default).
+      Parametrized by Lame coefficients.
+      @*/
     sub_command
       ("compute isotropic linearized Von Mises or Tresca", 4, 5, 0, 1,
        std::string varname = in.pop().to_string();
@@ -506,11 +701,47 @@ void gf_model_get(getfemint::mexargs_in& m_in,
        out.pop().from_dcvector(VMM);
        );
 
+    /*@GET V = ('compute isotropic linearized Von Mises pstrain', @str varname, @str data_E, @str data_nu, @tmf mf_vm)
+      Compute the Von-Mises stress  of a displacement field for isotropic
+      linearized elasticity in 3D or in 2D with plane strain assumption.
+      Parametrized by Young modulus and Poisson ratio.
+      @*/
+    sub_command
+      ("compute isotropic linearized Von Mises pstrain", 4, 4, 0, 1,
+       std::string varname = in.pop().to_string();
+       std::string data_E = in.pop().to_string();
+       std::string data_nu = in.pop().to_string();
+       getfemint_mesh_fem *gfi_mf = in.pop().to_getfemint_mesh_fem();
+
+       getfem::model_real_plain_vector VMM((gfi_mf->mesh_fem()).nb_dof());
+       getfem::compute_isotropic_linearized_Von_Mises_pstrain
+       (md->model(), varname, data_E, data_nu, gfi_mf->mesh_fem(), VMM);
+       out.pop().from_dcvector(VMM);
+       );
+
+    /*@GET V = ('compute isotropic linearized Von Mises pstress', @str varname, @str data_E, @str data_nu, @tmf mf_vm)
+      Compute the Von-Mises stress  of a displacement field for isotropic
+      linearized elasticity in 3D or in 2D with plane stress assumption.
+      Parametrized by Young modulus and Poisson ratio.
+      @*/
+    sub_command
+      ("compute isotropic linearized Von Mises pstress", 4, 4, 0, 1,
+       std::string varname = in.pop().to_string();
+       std::string data_E = in.pop().to_string();
+       std::string data_nu = in.pop().to_string();
+       getfemint_mesh_fem *gfi_mf = in.pop().to_getfemint_mesh_fem();
+
+       getfem::model_real_plain_vector VMM((gfi_mf->mesh_fem()).nb_dof());
+       getfem::compute_isotropic_linearized_Von_Mises_pstress
+       (md->model(), varname, data_E, data_nu, gfi_mf->mesh_fem(), VMM);
+       out.pop().from_dcvector(VMM);
+       );
 
     /*@GET V = ('compute Von Mises or Tresca', @str varname, @str lawname, @str dataname, @tmf mf_vm[, @str version])
       Compute on `mf_vm` the Von-Mises stress or the Tresca stress of a field
       for nonlinear elasticity in 3D. `lawname` is the constitutive law which
-      could be 'SaintVenant Kirchhoff', 'Mooney Rivlin' or 'Ciarlet Geymonat'.
+      could be 'SaintVenant Kirchhoff', 'Mooney Rivlin', 'neo Hookean' or
+      'Ciarlet Geymonat'.
       `dataname` is a vector of parameters for the constitutive law. Its length
       depends on the law. It could be a short vector of constant values or a
       vector field described on a finite element method for variable coefficients.
@@ -540,10 +771,34 @@ void gf_model_get(getfemint::mexargs_in& m_in,
        out.pop().from_dcvector(VMM);
        );
 
+    /*@GET V = ('finite strain elasticity Von Mises', @str varname, @str lawname, @str params, @tmf mf_vm[, @int region])
+      Compute on `mf_vm` the Von-Mises stress of a field `varname`
+      for nonlinear elasticity in 3D. `lawname` is the constitutive law which
+      should be a valid name. `params` are the parameters law. It could be
+      a short vector of constant values or may depend on data or variables
+      of the model.
+      Uses the high-level generic assembly.
+ @*/
+    sub_command
+      ("finite strain elasticity Von Mises", 4, 5, 0, 1,
+       std::string varname = in.pop().to_string();
+       std::string lawname = in.pop().to_string();
+       std::string params = in.pop().to_string();
+       getfemint_mesh_fem *gfi_mf = in.pop().to_getfemint_mesh_fem();
+       std::string stresca = "Von Mises";
+       size_type rg = size_type(-1);
+       if (in.remaining()) rg = in.pop().to_integer();
+       getfem::model_real_plain_vector VMM((gfi_mf->mesh_fem()).nb_dof());
+       getfem::finite_strain_elasticity_Von_Mises
+       (md->model(), varname, lawname, params,  gfi_mf->mesh_fem(), VMM, rg);
+       out.pop().from_dcvector(VMM);
+       );
+
     /*@GET V = ('compute second Piola Kirchhoff tensor', @str varname, @str lawname, @str dataname, @tmf mf_sigma)
       Compute on `mf_sigma` the second Piola Kirchhoff stress tensor of a field
       for nonlinear elasticity in 3D. `lawname` is the constitutive law which
-      could be 'SaintVenant Kirchhoff', 'Mooney Rivlin' or 'Ciarlet Geymonat'.
+      could be 'SaintVenant Kirchhoff', 'Mooney Rivlin', 'neo Hookean' or
+      'Ciarlet Geymonat'.
       `dataname` is a vector of parameters for the constitutive law. Its length
       depends on the law. It could be a short vector of constant values or a
       vector field described on a finite element method for variable
@@ -573,7 +828,7 @@ void gf_model_get(getfemint::mexargs_in& m_in,
        );
 
 
-    /*@GET V = ('compute plasticity Von Mises or Tresca', @str datasigma, @tmf mf_vm[, @str version])
+    /*@GET V = ('compute elastoplasticity Von Mises or Tresca', @str datasigma, @tmf mf_vm[, @str version])
       Compute on `mf_vm` the Von-Mises or the Tresca stress of a field for plasticity and return it into the vector V.
       `datasigma` is a vector which contains the stress constraints values supported by the mesh.
       `version` should be  'Von_Mises' or 'Tresca' ('Von_Mises' is the default).@*/
@@ -600,7 +855,7 @@ void gf_model_get(getfemint::mexargs_in& m_in,
 
 
 
-        /*@GET ('compute plasticity constraints', @tmim mim, @str varname, @str projname, @str datalambda, @str datamu, @str datathreshold, @str datasigma)
+    /*@GET ('elastoplasticity next iter', @tmim mim, @str varname, @str projname, @str datalambda, @str datamu, @str datathreshold, @str datasigma)
       Compute and save the stress constraints sigma for other hypothetical iterations.
       'mim' is the integration method to use for the computation.
       'varname' is the main variable of the problem.
@@ -649,7 +904,40 @@ void gf_model_get(getfemint::mexargs_in& m_in,
        out.pop().from_dcvector(plast);
        );
 
+     /*@GET V = ('sliding data group name of large sliding contact brick', @int indbrick)
+      Gives the name of the group of variables corresponding to the
+      sliding data for an existing large sliding contact brick.@*/
+      sub_command
+      ("sliding data group name of large sliding contact brick", 1, 1, 0, 1,
+       size_type ind = in.pop().to_integer() - config::base_index();
+       std::string name
+       = sliding_data_group_name_of_large_sliding_contact_brick(md->model(),
+                                                                ind);
+       out.pop().from_string(name.c_str());
+       );
 
+     /*@GET V = ('displacement group name of large sliding contact brick', @int indbrick)
+      Gives the name of the group of variables corresponding to the
+      sliding data for an existing large sliding contact brick.@*/
+      sub_command
+      ("displacement group name of large sliding contact brick", 1, 1, 0, 1,
+       size_type ind = in.pop().to_integer() - config::base_index();
+       std::string name
+       = displacement_group_name_of_large_sliding_contact_brick(md->model(),
+                                                                ind);
+       out.pop().from_string(name.c_str());
+       );
+
+     /*@GET V = ('transformation name of large sliding contact brick', @int indbrick)
+      Gives the name of the group of variables corresponding to the
+      sliding data for an existing large sliding contact brick.@*/
+      sub_command
+      ("transformation name of large sliding contact brick", 1, 1, 0, 1,
+       size_type ind = in.pop().to_integer() - config::base_index();
+       std::string name
+       = transformation_name_of_large_sliding_contact_brick(md->model(), ind);
+       out.pop().from_string(name.c_str());
+       );
 
     /*@GET M = ('matrix term', @int ind_brick, @int ind_term)
       Gives the matrix term ind_term of the brick ind_brick if it exists
