@@ -1,10 +1,10 @@
 /*===========================================================================
- 
- Copyright (C) 2000-2012 Yves Renard
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2000-2015 Yves Renard
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -16,18 +16,22 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
 ===========================================================================*/
 
 
 #include "getfem/dal_singleton.h"
 #include "getfem/dal_tree_sorted.h"
-#include "getfem/dal_naming_system.h"
 #include "getfem/bgeot_geometric_trans.h"
 #include "getfem/bgeot_poly_composite.h"
 
 namespace bgeot {
 
+  void geometric_trans::compute_K_matrix
+    (const base_matrix &G, const base_matrix &pc, base_matrix &K) const{
+    gmm::mult(G, pc, K);
+  }
+
   const base_node& geotrans_interpolation_context::xref() const {
     if (!have_xref()) {
       if (pspt_) xref_ = (*pspt_)[ii_];
@@ -64,17 +68,18 @@ namespace bgeot {
       size_type P = pgt_->structure()->dim();
       K_.resize(N(), P);
       if (have_pgp()) {
+        GMM_ASSERT1(ii_ < pgp_->get_point_tab().size(),
+                    "Invalid index " << ii_ << " should be < "
+                    << pgp_->get_point_tab().size());
 
         if (&pgp_->grad(ii_) == 0) { cerr << "OULA!! " << ii_ << "\n"; }
         else if (pgp_->grad(ii_).size() == 0) { cerr << "OUCH\n"; }
 
-        assert(ii_ < pgp_->get_point_tab().size());
-
-        gmm::mult(G(), pgp_->grad(ii_), K_);
+        pgt()->compute_K_matrix(G(), pgp_->grad(ii_), K_);
       } else {
         base_matrix pc(pgt()->nb_points(), P);
         pgt()->poly_vector_grad(xref(), pc);
-        gmm::mult(G(),pc,K_);
+        pgt()->compute_K_matrix(G(), pc, K_);
       }
     }
     return K_;
@@ -145,8 +150,9 @@ namespace bgeot {
     if (have_B() && !pgt()->is_linear()) { B_.resize(0,0); }
     if (have_B3() && !pgt()->is_linear()) {
       B3_.resize(0,0); B32_.resize(0,0); }
+    if (J_ >= scalar_type(0) && !pgt()->is_linear()) J_ = scalar_type(-1);
     xref_.clear(); xreal_.clear();
-    ii_=ii__;  J_ = scalar_type(-1);
+    ii_=ii__;
   }
 
   void geotrans_interpolation_context::set_xref(const base_node& P) {
@@ -155,7 +161,8 @@ namespace bgeot {
     if (have_B() && !pgt()->is_linear()) { B_.resize(0,0); }
     if (have_B3() && !pgt()->is_linear()) {
       B3_.resize(0,0); B32_.resize(0,0); }
-    xreal_.clear(); ii_ = size_type(-1); J_ = scalar_type(-1); pspt_ = 0;
+    if (J_ >= scalar_type(0) && !pgt()->is_linear()) J_ = scalar_type(-1);
+    xreal_.clear(); ii_ = size_type(-1); pspt_ = 0;
   }
 
   geotrans_interpolation_context::geotrans_interpolation_context() :
@@ -176,8 +183,6 @@ namespace bgeot {
     ii_(size_type(-1)), J_(-1) {}
 
 
-  typedef dal::naming_system<geometric_trans>::param_list gt_param_list;
-
   base_node geometric_trans::transform(const base_node &pt,
                                        const base_matrix &G) const {
     size_type N = G.nrows(), k = nb_points();
@@ -217,7 +222,7 @@ namespace bgeot {
     virtual void poly_vector_val(const base_node &pt, base_vector &val) const {
       val.resize(nb_points());
       for (size_type k = 0; k < nb_points(); ++k)
-        val[k] = trans[k].eval(pt.begin());
+        val[k] = to_scalar(trans[k].eval(pt.begin()));
     }
 
     virtual void poly_vector_val(const base_node &pt, const convex_ind_ct &ind_ct,
@@ -225,7 +230,7 @@ namespace bgeot {
       size_type nb_funcs=ind_ct.size();
       val.resize(nb_funcs);
       for (size_type k = 0; k < nb_funcs; ++k)
-        val[k] = trans[ind_ct[k]].eval(pt.begin());
+        val[k] = to_scalar(trans[ind_ct[k]].eval(pt.begin()));
     }
 
     virtual void poly_vector_grad(const base_node &pt, base_matrix &pc) const {
@@ -235,11 +240,12 @@ namespace bgeot {
         for (dim_type n = 0; n < dim(); ++n) {
           PP = trans[i];
           PP.derivative(n);
-          pc(i, n) = PP.eval(pt.begin());
+          pc(i, n) = to_scalar(PP.eval(pt.begin()));
         }
     }
 
-    virtual void poly_vector_grad(const base_node &pt, const convex_ind_ct &ind_ct,
+    virtual void poly_vector_grad(const base_node &pt,
+				  const convex_ind_ct &ind_ct,
                                   base_matrix &pc) const {
       FUNC PP;
       size_type nb_funcs=ind_ct.size();
@@ -248,7 +254,7 @@ namespace bgeot {
         for (dim_type n = 0; n < dim(); ++n) {
           PP = trans[ind_ct[i]];
           PP.derivative(n);
-          pc(i, n) = PP.eval(pt.begin());
+          pc(i, n) = to_scalar(PP.eval(pt.begin()));
         }
     }
 
@@ -260,7 +266,7 @@ namespace bgeot {
           QP = trans[i]; QP.derivative(n);
           for (dim_type m = 0; m <= n; ++m) {
             PP = QP; PP.derivative(m);
-            pc(i, n*dim()+m) = pc(i, m*dim()+n) = PP.eval(pt.begin());
+            pc(i, n*dim()+m) = pc(i, m*dim()+n) = to_scalar(PP.eval(pt.begin()));
           }
         }
     }
@@ -469,8 +475,95 @@ namespace bgeot {
     return parallelepiped_linear_geotrans(n);
   }
 
+
+  /* ******************************************************************** */
+  /*	Incomplete Q2 geometric transformation for n=2 or 3.              */
+  /* ******************************************************************** */
+  /* By Yao Koutsawa  <yao.koutsawa@tudor.lu> 2012-12-10                  */
+
+  struct Q2_incomplete_trans_: public poly_geometric_trans  {
+    Q2_incomplete_trans_(dim_type nc) {
+      cvr = Q2_incomplete_reference(nc);
+      size_type R = cvr->structure()->nb_points();
+      is_lin = false;
+      complexity_ = 2;
+      trans.resize(R);
+      
+      if (nc == 2) {
+        std::stringstream s
+          ( "1 - 2*x^2*y - 2*x*y^2 + 2*x^2 + 5*x*y + 2*y^2 - 3*x - 3*y;"
+            "4*(x^2*y - x^2 - x*y + x);"
+            "2*x*y*y - 2*x*x*y + 2*x*x - x*y - x;"
+            "4*(x*y*y - x*y - y*y + y);"
+            "4*(x*y - x*y*y);"
+            "2*x*x*y - 2*x*y*y - x*y + 2*y*y - y;"
+            "4*(x*y - x*x*y);"
+            "2*x*x*y + 2*x*y*y - 3*x*y;");
+        
+        for (int i = 0; i < 8; ++i)
+          trans[i] = bgeot::read_base_poly(2, s);
+      } else {
+        std::stringstream s
+          ("1 + 2*x^2*y*z + 2*x*y^2*z + 2*x*y*z^2"
+             " - 2*x^2*y - 2*x^2*z - 2*x*y^2 - 2*y^2*z - 2*y*z^2 - 2*x*z^2 - 7*x*y*z"
+             " + 2*x^2 + 2*y^2 + 2*z^2 + 5*y*z + 5*x*z + 5*x*y - 3*x - 3*y - 3*z;"
+           "4*( - x^2*y*z + x*y*z + x^2*z - x*z + x^2*y - x*y - x^2 + x);"
+           "2*x^2*y*z - 2*x*y^2*z - 2*x*y*z^2"
+             " - 2*x^2*y - 2*x^2*z + 2*x*y^2 + 2*x*z^2 + 3*x*y*z + 2*x^2 - x*y - x*z - x;"
+           "4*( - x*y^2*z + x*y^2 + y^2*z + x*y*z - x*y - y^2 - y*z + y);"
+           "4*(x*y^2*z - x*y^2 - x*y*z + x*y);"
+           " - 2*x^2*y*z + 2*x*y^2*z - 2*x*y*z^2"
+             " + 2*x^2*y - 2*x*y^2 - 2*y^2*z + 2*y*z^2 + 3*x*y*z - x*y + 2*y^2 - y*z - y;"
+           "4*(x^2*y*z - x^2*y - x*y*z + x*y);"
+           " - 2*x^2*y*z - 2*x*y^2*z + 2*x*y*z^2 + 2*x^2*y + 2*x*y^2 + x*y*z - 3*x*y;"
+           "4*( - x*y*z^2 + x*z^2 + y*z^2 + x*y*z - x*z - y*z - z^2 + z);"
+           "4*(x*y*z^2 - x*y*z - x*z^2 + x*z);"
+           "4*(x*y*z^2 - x*y*z - y*z^2 + y*z);"
+           "4*( - x*y*z^2 + x*y*z);"
+           " - 2*x^2*y*z - 2*x*y^2*z + 2*x*y*z^2"
+             " + 2*x^2*z + 2*y^2*z - 2*x*z^2 - 2*y*z^2 + 3*x*y*z - x*z - y*z + 2*z^2 - z;"
+           "4*(x^2*y*z - x^2*z - x*y*z + x*z);"
+           " - 2*x^2*y*z + 2*x*y^2*z - 2*x*y*z^2 + 2*x^2*z + 2*x*z^2 + x*y*z - 3*x*z;"
+           "4*(x*y^2*z - y^2*z - x*y*z + y*z);"
+           "4*( - x*y^2*z + x*y*z);"
+           "2*x^2*y*z - 2*x*y^2*z - 2*x*y*z^2 + 2*y^2*z + 2*y*z^2 + x*y*z - 3*y*z;"
+           "4*( - x^2*y*z + x*y*z);"
+           "2*x^2*y*z + 2*x*y^2*z + 2*x*y*z^2 - 5*x*y*z;");
+        
+        for (int i = 0; i < 20; ++i)
+          trans[i] = bgeot::read_base_poly(3, s);
+      }
+      fill_standard_vertices();
+    }
+  };
+  
+  static pgeometric_trans
+    Q2_incomplete_gt(gt_param_list& params,
+                     std::vector<dal::pstatic_stored_object> &dependencies) {
+    GMM_ASSERT1(params.size() == 1, "Bad number of parameters : " << params.size() << " should be 1.");
+    GMM_ASSERT1(params[0].type() == 0, "Bad type of parameters");
+    int n = int(::floor(params[0].num() + 0.01));
+    GMM_ASSERT1(n == 2 || n == 3, "Bad parameter, expected value 2 or 3");
+    
+    dependencies.push_back(Q2_incomplete_reference(dim_type(n)));
+    return new Q2_incomplete_trans_(dim_type(n));
+  }
+  
+  pgeometric_trans Q2_incomplete_geotrans(dim_type nc) {
+    static pgeometric_trans pgt = 0;
+    std::stringstream name;
+    name << "GT_Q2_INCOMPLETE(" << nc << ")";
+    pgt = geometric_trans_descriptor(name.str());
+    return pgt;
+  }
+
+
+  /* ******************************************************************** */
+  /*    Misc function.                                                    */
+  /* ******************************************************************** */
+
   /* norm of returned vector is the ratio between the face surface on
-     the real element and the face surface on the reference element
+     the reference element and the face surface on the real element.
      IT IS NOT UNITARY
 
      pt is the position of the evaluation point on the reference element
@@ -492,7 +585,6 @@ namespace bgeot {
                       size_type face) {
     GMM_ASSERT1(c.G().ncols() == c.pgt()->nb_points(), "dimensions mismatch");
     base_small_vector up = c.pgt()->normals()[face];
-    base_small_vector un(c.N());
     size_type P = c.pgt()->structure()->dim();
 
     base_matrix baseP(P, P);
@@ -506,7 +598,7 @@ namespace bgeot {
     base_matrix baseN(c.N(), P);
     gmm::mult(c.B(), baseP, baseN);
 
-    /* modified gram-schmidt */
+    /* Modified Gram-Schmidt */
     for (size_type k=0; k < P; ++k) {
       for (size_type l=0; l < k; ++l) {
         gmm::add(gmm::scaled(gmm::mat_col(baseN,l),
@@ -517,9 +609,9 @@ namespace bgeot {
       gmm::scale(gmm::mat_col(baseN,k),
                  1./gmm::vect_norm2(gmm::mat_col(baseN,k)));
     }
-    /* TODO: for cases where P < N,
-       complete the basis */
-    /* ensure that the baseN is direct */
+    /* TODO: for cases where P < N, complete the basis */
+
+    /* Ensure that the baseN is direct */
     if (c.N() == P && c.N()>1 && gmm::lu_det(baseN) < 0) {
       gmm::scale(gmm::mat_col(baseN,1),-1.);
     }
@@ -527,6 +619,8 @@ namespace bgeot {
   }
 
 
+
+
   /* ******************************************************************** */
   /*    Naming system                                                     */
   /* ******************************************************************** */
@@ -541,9 +635,16 @@ namespace bgeot {
       add_suffix("PRODUCT", product_gt);
       add_suffix("LINEAR_PRODUCT", linear_product_gt);
       add_suffix("LINEAR_QK", linear_qk);
+      add_suffix("Q2_INCOMPLETE", Q2_incomplete_gt);
     }
   };
 
+  void add_geometric_trans_name
+    (std::string name, dal::naming_system<geometric_trans>::pfunction f) {
+    dal::singleton<geometric_trans_naming_system>::instance().add_suffix(name,
+									 f);
+  }
+
   pgeometric_trans geometric_trans_descriptor(std::string name) {
     size_type i=0;
     return dal::singleton<geometric_trans_naming_system>::instance().method(name, i);
