@@ -1,11 +1,11 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
- 
- Copyright (C) 2000-2012 Yves Renard
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2000-2015 Yves Renard
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -17,7 +17,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
  As a special exception, you  may use  this file  as it is a part of a free
  software  library  without  restriction.  Specifically,  if   other  files
  instantiate  templates  or  use macros or inline functions from this file,
@@ -26,7 +26,7 @@
  to be covered  by the GNU Lesser General Public License.  This   exception
  does not  however  invalidate  any  other  reasons why the executable file
  might be covered by the GNU Lesser General Public License.
- 
+
 ===========================================================================*/
 
 /** @file bgeot_geometric_trans.h
@@ -41,6 +41,7 @@
 #include <set>
 #include "bgeot_config.h"
 #include "bgeot_convex_ref.h"
+#include "getfem/dal_naming_system.h"
 
 namespace bgeot {
 
@@ -103,7 +104,7 @@ namespace bgeot {
    *  <h3>Inverse transformation and pseudo-inverse</h3>
    *     to do ...
    */
-  class geometric_trans : virtual public dal::static_stored_object {
+  class APIDECL geometric_trans : virtual public dal::static_stored_object {
   protected :
 
     bool is_lin;
@@ -140,6 +141,8 @@ namespace bgeot {
                                   base_matrix &val) const = 0;
     /// Gives the hessian of the functions vector at a certain point.
     virtual void poly_vector_hess(const base_node &pt, base_matrix &val) const = 0;
+    /// compute K matrix from multiplication of G with gradient
+    virtual void compute_K_matrix(const base_matrix &G, const base_matrix &pc, base_matrix &K) const;
     /// Gives the number of vertices.
     size_type nb_vertices(void) const { return vertices_.size(); }
     /// Gives the indices of vertices between the nodes.
@@ -204,35 +207,38 @@ namespace bgeot {
    */
   //@{
 
-  pgeometric_trans simplex_geotrans(size_type n, short_type k);
-  pgeometric_trans parallelepiped_geotrans(size_type n, short_type k);
-  pgeometric_trans parallelepiped_linear_geotrans(size_type n);
-  pgeometric_trans prism_geotrans(size_type n, short_type k);
-  pgeometric_trans prism_linear_geotrans(size_type n);
-  pgeometric_trans product_geotrans(pgeometric_trans pg1,
+  pgeometric_trans APIDECL simplex_geotrans(size_type n, short_type k);
+  pgeometric_trans APIDECL parallelepiped_geotrans(size_type n, short_type k);
+  pgeometric_trans APIDECL parallelepiped_linear_geotrans(size_type n);
+  pgeometric_trans APIDECL prism_geotrans(size_type n, short_type k);
+  pgeometric_trans APIDECL prism_linear_geotrans(size_type n);
+  pgeometric_trans APIDECL product_geotrans(pgeometric_trans pg1,
                                     pgeometric_trans pg2);
-  pgeometric_trans linear_product_geotrans(pgeometric_trans pg1,
+  pgeometric_trans APIDECL linear_product_geotrans(pgeometric_trans pg1,
                                            pgeometric_trans pg2);
+  pgeometric_trans APIDECL Q2_incomplete_geotrans(dim_type nc);
 
   /**
      Get the geometric transformation from its string name.
      @see name_of_geometric_trans
   */
-  pgeometric_trans geometric_trans_descriptor(std::string name);
+  pgeometric_trans APIDECL geometric_trans_descriptor(std::string name);
   /**
      Get the string name of a geometric transformation.
 
-  List of possible names:
-   * GT_PK(N,K)   : Transformation on simplexes, dim N, degree K
-   * GT_QK(N,K)   : Transformation on parallelepipeds, dim N, degree K
-   * GT_PRISM(N,K)          : Transformation on prisms, dim N, degree K
-   * GT_PRODUCT(a,b)        : tensorial product of two transformations
-   * GT_LINEAR_PRODUCT(a,b) : Linear tensorial product of two transformations
-   * GT_LINEAR_QK(N) : shortcut for GT_LINEAR_PRODUCT(GT_LINEAR_QK(N-1),
-   *                                                  GT_PK(1,1))
+     List of possible names:
+     GT_PK(N,K)   : Transformation on simplexes, dim N, degree K
+     
+     GT_QK(N,K)   : Transformation on parallelepipeds, dim N, degree K
+     GT_PRISM(N,K)          : Transformation on prisms, dim N, degree K
+     GT_Q2_INCOMPLETE(N)    : Q2 incomplete transformation in dim N=2 or 3.
+     GT_PRODUCT(a,b)        : tensorial product of two transformations
+     GT_LINEAR_PRODUCT(a,b) : Linear tensorial product of two transformations
+     GT_LINEAR_QK(N) : shortcut for GT_LINEAR_PRODUCT(GT_LINEAR_QK(N-1),
+                                                      GT_PK(1,1))
    */
 
-  std::string name_of_geometric_trans(pgeometric_trans p);
+  std::string APIDECL name_of_geometric_trans(pgeometric_trans p);
 
   /** norm of returned vector is the ratio between the face surface on
    *  the real element and the face surface on the reference element
@@ -240,14 +246,14 @@ namespace bgeot {
    *
    *  pt is the position of the evaluation point on the reference element
    */
-  base_small_vector compute_normal(const geotrans_interpolation_context& c,
+  base_small_vector APIDECL compute_normal(const geotrans_interpolation_context& c,
                                    size_type face);
 
   /** return the local basis (i.e. the normal in the first column, and the
    *  tangent vectors in the other columns
    */
   base_matrix
-  compute_local_basis(const geotrans_interpolation_context& c,
+  APIDECL compute_local_basis(const geotrans_interpolation_context& c,
                       size_type face);
     //@}
 
@@ -265,7 +271,7 @@ namespace bgeot {
    *  repetitive evaluation of a geometric transformations on a set of
    *  points "pspt" in the reference convex which do not change.
    */
-  class geotrans_precomp_ : virtual public dal::static_stored_object {
+  class APIDECL geotrans_precomp_ : virtual public dal::static_stored_object {
   protected:
     pgeometric_trans pgt;
     pstored_point_tab pspt;  /* a set of points in the reference elt*/
@@ -333,7 +339,7 @@ namespace bgeot {
          itk != G.end(); ++itk, ++k)
       gmm::add(gmm::scaled(*itk, c[j][k]), pt);
     GMM_ASSERT1(k == pgt->nb_points(),
-                "Wrong number of points in tranformation");
+                "Wrong number of points in transformation");
   }
 
   template <typename CONT>
@@ -346,14 +352,14 @@ namespace bgeot {
     }
   }
 
-  void delete_geotrans_precomp(pgeotrans_precomp pgp);
+  void APIDECL delete_geotrans_precomp(pgeotrans_precomp pgp);
 
   /**
    *  The object geotrans_precomp_pool Allow to allocate a certain number
    *  of geotrans_precomp and automatically delete them when it is
    *  deleted itself.
    */
-  class geotrans_precomp_pool {
+  class APIDECL geotrans_precomp_pool {
     std::set<pgeotrans_precomp> precomps;
 
   public :
@@ -379,9 +385,9 @@ namespace bgeot {
      the xreal will be computed if needed as long as pgp+ii is known).
      See also fem_interpolation_context in getfem_fem.h.
      The name of member data, and the computations done by this structure
-     are heavily described in the Getfem++ Kernel Documentation.
+     are heavily described in the GetFEM++ Kernel Documentation.
   */
-  class geotrans_interpolation_context {
+  class APIDECL geotrans_interpolation_context {
     mutable base_node xref_; /** reference point */
     mutable base_node xreal_; /** transformed point */
     const base_matrix *G_; /** pointer to the matrix of real nodes of the convex */
@@ -438,6 +444,15 @@ namespace bgeot {
                                    const base_matrix& G__);
   };
 
+  /* Function allowing the add of an geometric transformation method outwards
+     of getfem_integration.cc */
+  
+  typedef dal::naming_system<geometric_trans>::param_list gt_param_list;
+
+  void APIDECL add_geometric_trans_name
+  (std::string name, dal::naming_system<geometric_trans>::pfunction f);
+
+
 }  /* end of namespace bgeot.                                             */
 
 
