@@ -1,11 +1,11 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
- 
- Copyright (C) 2003-2012 Julien Pommier
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2003-2015 Julien Pommier
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -17,7 +17,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
  As a special exception, you  may use  this file  as it is a part of a free
  software  library  without  restriction.  Specifically,  if   other  files
  instantiate  templates  or  use macros or inline functions from this file,
@@ -26,7 +26,7 @@
  to be covered  by the GNU Lesser General Public License.  This   exception
  does not  however  invalidate  any  other  reasons why the executable file
  might be covered by the GNU Lesser General Public License.
- 
+
 ===========================================================================*/
 
 /**@file getfem_assembling_tensors.h
@@ -220,50 +220,55 @@ namespace getfem {
     void reinit_() {
       mti = multi_tensor_iterator(child(0).tensor(),true);
     }
+
     void exec_(size_type cv, dim_type) {
       tensor_ranges r;
       std::vector< tensor_strides > str;
       vdim.build_strides_for_cv(cv, r, str);
       if (child(0).ranges() != r) {
-	ASM_THROW_TENSOR_ERROR("can't output a tensor of dimensions "
-			       << child(0).ranges() << 
-			       " into an output array of size " << r);
+        ASM_THROW_TENSOR_ERROR("can't output a tensor of dimensions "
+          << child(0).ranges() << 
+          " into an output array of size " << r);
       }
       mti.rewind();
       if (pmf && pmf->is_reduced()) {
-	do {
-	  size_type nb_dof =  pmf->nb_dof();
-	  dim_type qqdim = dim_type(gmm::vect_size(v) / nb_dof);
-	 
-	  if (qqdim == 1) {
-	    size_type i = 0;
-	    for (dim_type j=0; j < mti.ndim(); ++j) i += str[j][mti.index(j)];
-	    gmm::add(gmm::scaled(gmm::mat_row(pmf->extension_matrix(), i),
-				 mti.p(0)), v);
-	  }
-	  else {
-	    GMM_ASSERT1(false, "To be verified ... ");
-	    size_type i = 0;
-	    for (dim_type j=0; j < mti.ndim(); ++j) i += str[j][mti.index(j)];
-	    gmm::add(gmm::scaled(gmm::mat_row(pmf->extension_matrix(),i/qqdim),
-				 mti.p(0)),
-		     gmm::sub_vector(v, gmm::sub_slice(i%qqdim,nb_dof,qqdim)));
-	  }
-	} while (mti.qnext1());
+        if ( pmf->nb_dof() != 0)
+        {
+          do {
+            size_type nb_dof =  pmf->nb_dof();
+            dim_type qqdim = dim_type(gmm::vect_size(v) / nb_dof);
+
+            if (qqdim == 1) {
+              size_type i = 0;
+              for (dim_type j=0; j < mti.ndim(); ++j) i += str[j][mti.index(j)];
+              gmm::add(gmm::scaled(gmm::mat_row(pmf->extension_matrix(), i),
+                mti.p(0)), v);
+            }
+            else {
+              GMM_ASSERT1(false, "To be verified ... ");
+              size_type i = 0;
+              for (dim_type j=0; j < mti.ndim(); ++j) i += str[j][mti.index(j)];
+              gmm::add(gmm::scaled(gmm::mat_row(pmf->extension_matrix(),i/qqdim),
+                mti.p(0)),
+                gmm::sub_vector(v, gmm::sub_slice(i%qqdim,nb_dof,qqdim)));
+            }
+          } while (mti.qnext1());
+        }
       }
       else {
-	do {
-	  typename gmm::linalg_traits<VEC>::iterator it = gmm::vect_begin(v);
-	  for (dim_type j = 0; j < mti.ndim(); ++j) it += str[j][mti.index(j)];
-	  *it += mti.p(0);
-	} while (mti.qnext1());
+        do {
+          typename gmm::linalg_traits<VEC>::iterator it = gmm::vect_begin(v);
+          for (dim_type j = 0; j < mti.ndim(); ++j) it += str[j][mti.index(j)];
+          *it += mti.p(0);
+        } while (mti.qnext1());
       }
     }
   };
 
   template <typename MAT, typename ROW, typename COL>
-  void asmrankoneupdate(MAT &m, const ROW &row, const COL &col,
+  void asmrankoneupdate(const MAT &m_, const ROW &row, const COL &col,
 			scalar_type r) {
+    MAT &m = const_cast<MAT &>(m_);
     typename gmm::linalg_traits<ROW>::const_iterator itr = row.begin();
     for (; itr != row.end(); ++itr) {
       typename gmm::linalg_traits<COL>::const_iterator itc = col.begin();
@@ -273,13 +278,15 @@ namespace getfem {
   }
   
   template <typename MAT, typename ROW>
-  void asmrankoneupdate(MAT &m, const ROW &row, size_type j, scalar_type r) {
+  void asmrankoneupdate(const MAT &m_, const ROW &row, size_type j, scalar_type r) {
+    MAT &m = const_cast<MAT &>(m_);
     typename gmm::linalg_traits<ROW>::const_iterator itr = row.begin();
     for (; itr != row.end(); ++itr) m(itr.index(), j) += (*itr) * r;
   }
 
   template <typename MAT, typename COL>
-  void asmrankoneupdate(MAT &m, size_type j, const COL &col, scalar_type r) {
+  void asmrankoneupdate(const MAT &m_, size_type j, const COL &col, scalar_type r) {
+    MAT &m = const_cast<MAT &>(m_);
     typename gmm::linalg_traits<COL>::const_iterator itc = col.begin();
     for (; itc != col.end(); ++itc) m(j, itc.index()) += (*itc) * r;
   }
@@ -326,15 +333,7 @@ namespace getfem {
 				     mf_r.ind_basic_dof_of_element(cv).end());
       std::vector<size_type> cvdof_c(mf_c.ind_basic_dof_of_element(cv).begin(),
 				     mf_c.ind_basic_dof_of_element(cv).end());
-      /*mti.rewind();
-      do {
-	if (mti.p(0)) {
-	  size_type dof_i = cvdof_r[mti.index(0)];
-	  size_type dof_j = cvdof_c[mti.index(1)];
-	  m(dof_i, dof_j) += mti.p(0);
-	}
-      } while (mti.qnext1());
-      */
+
       if (it.size() == 0) {
 	mti.rewind();
 	do {
@@ -458,7 +457,7 @@ namespace getfem {
       if (sz == 0)
 	ASM_THROW_TENSOR_ERROR("can't create a vector of size " << r);
       asm_vec<VEC> v(new VEC(sz));
-      push_back(v); return &this->back();
+      this->push_back(v); return &this->back();
     }
     ~vec_factory() { 
       for (size_type i=0; i < this->size(); ++i) {
