@@ -1,11 +1,11 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
- 
- Copyright (C) 1999-2012 Yves Renard
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 1999-2015 Yves Renard
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -17,7 +17,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
  As a special exception, you  may use  this file  as it is a part of a free
  software  library  without  restriction.  Specifically,  if   other  files
  instantiate  templates  or  use macros or inline functions from this file,
@@ -26,7 +26,7 @@
  to be covered  by the GNU Lesser General Public License.  This   exception
  does not  however  invalidate  any  other  reasons why the executable file
  might be covered by the GNU Lesser General Public License.
- 
+
 ===========================================================================*/
 
 /**@file getfem_fem.h
@@ -49,8 +49,8 @@
    - "FEM_QK_DISCONTINUOUS(N,K,alpha)" : discontinuous Lagrange
    element QK on a parallelepiped.
 
-   - "FEM_Q2_INCOMPLETE" : incomplete Q2 element with 8 dof (serendipity
-                           Quad 8 element)
+   - "FEM_Q2_INCOMPLETE(N)" : incomplete Q2 elements with 8 and 20 dof
+                              (serendipity Quad 8 and Hexa 20 elements)
 
    - "FEM_PK_PRISM(N,K)" : classical Lagrange element PK on a prism.
 
@@ -379,6 +379,14 @@ namespace getfem {
                             const CVEC& coeff, VMAT &val,
                             dim_type Qdim) const;
 
+    /** Interpolation of the divergence. The output is stored in the
+        scalar @c val.
+     */
+    template<typename CVEC>
+    void interpolation_diverg
+      (const fem_interpolation_context& c, const CVEC& coeff,
+       typename gmm::linalg_traits<CVEC>::value_type &val) const;
+
     /** Give the value of all components of the base functions at the
      *  point x of the reference element. Basic function used essentially
      *  by fem_precomp.
@@ -488,6 +496,8 @@ namespace getfem {
     /// Gives the array of basic functions (components).
     const std::vector<FUNC> &base(void) const { return base_; }
     std::vector<FUNC> &base(void) { return base_; }
+    /** Evaluates at point x, all base functions and returns the result in
+        t(nb_base,target_dim) */
     void base_value(const base_node &x, base_tensor &t) const {
       bgeot::multi_index mi(2);
       mi[1] = target_dim(); mi[0] = short_type(nb_base(0));
@@ -495,8 +505,11 @@ namespace getfem {
       size_type R = nb_base_components(0);
       base_tensor::iterator it = t.begin();
       for (size_type  i = 0; i < R; ++i, ++it)
-        *it = base_[i].eval(x.begin());
+        *it = bgeot::to_scalar(base_[i].eval(x.begin()));
     }
+    /** Evaluates at point x, the gradient of all base functions w.r.t. the
+        reference element directions 0,..,dim-1 and returns the result in
+        t(nb_base,target_dim,dim) */
     void grad_base_value(const base_node &x, base_tensor &t) const {
       bgeot::multi_index mi(3);
       dim_type n = dim();
@@ -506,8 +519,11 @@ namespace getfem {
       base_tensor::iterator it = t.begin();
       for (dim_type j = 0; j < n; ++j)
         for (size_type i = 0; i < R; ++i, ++it)
-          { FUNC f = base_[i]; f.derivative(j); *it = f.eval(x.begin()); }
+          { FUNC f = base_[i]; f.derivative(j); *it = bgeot::to_scalar(f.eval(x.begin())); }
     }
+    /** Evaluates at point x, the hessian of all base functions w.r.t. the
+        reference element directions 0,..,dim-1 and returns the result in
+        t(nb_base,target_dim,dim,dim) */
     void hess_base_value(const base_node &x, base_tensor &t) const {
       bgeot::multi_index mi(4);
       dim_type n = dim();
@@ -521,7 +537,7 @@ namespace getfem {
           for (size_type i = 0; i < R; ++i, ++it) {
             FUNC f = base_[i];
             f.derivative(j); f.derivative(k);
-            *it = f.eval(x.begin());
+            *it = bgeot::to_scalar(f.eval(x.begin()));
           }
     }
 
@@ -667,8 +683,8 @@ namespace getfem {
     mutable base_matrix M_; // optional transformation matrix (for non tau-equivalent fems)
     pfem pf_;               // current fem
     pfem_precomp pfp_;      // optional fem_precomp_ (speed up the computations)
-    size_type convex_num_;  // the convex number (info needed by some specific FEMs)
-    size_type face_num_;    // Face number for boundary integration
+    size_type convex_num_;  // The element (convex) number
+    short_type face_num_;   // Face number for boundary integration
   public:
     /// true if a fem_precomp_ has been supplied.
     bool have_pfp() const { return pfp_ != 0; }
@@ -692,8 +708,11 @@ namespace getfem {
     const pfem pf() const { return pf_; }
     /** get the current convex number */
     size_type convex_num() const;
+    bool is_convex_num_valid() const;
     /** get the current face number */
-    size_type face_num() const;
+    short_type face_num() const;
+    /** On a face ? */
+    bool is_on_face() const;
     /** get the current fem_precomp_ */
     pfem_precomp pfp() const { return pfp_; }
     void set_pfp(pfem_precomp newpfp);
@@ -703,39 +722,41 @@ namespace getfem {
                               pfem_precomp pfp__, size_type ii__,
                               const base_matrix& G__,
                               size_type convex_num__,
-                              size_type face_num__ = size_type(-1));
+                              short_type face_num__ = short_type(-1));
     fem_interpolation_context(bgeot::pgeometric_trans pgt__,
                               pfem_precomp pfp__, size_type ii__,
                               const base_matrix& G__,
                               size_type convex_num__,
-                              size_type face_num__ = size_type(-1));
+                              short_type face_num__ = short_type(-1));
     fem_interpolation_context(bgeot::pgeometric_trans pgt__,
                               pfem pf__,
                               const base_node& xref__,
                               const base_matrix& G__,
                               size_type convex_num__,
-                              size_type face_num__ = size_type(-1));
+                              short_type face_num__ = short_type(-1));
   };
 
+  // IN : coeff(Qmult,nb_dof)
+  // OUT: val(Qdim), Qdim=target_dim*Qmult
+  // AUX: Z(nb_dof,target_dim)
   template <typename CVEC, typename VVEC>
   void virtual_fem::interpolation(const fem_interpolation_context& c,
                                   const CVEC& coeff, VVEC &val,
                                   dim_type Qdim) const {
     size_type Qmult = size_type(Qdim) / target_dim();
-    size_type R = nb_dof(c.convex_num());
+    size_type nbdof = nb_dof(c.convex_num());
     GMM_ASSERT1(gmm::vect_size(val) == Qdim, "dimensions mismatch");
-    GMM_ASSERT1(gmm::vect_size(coeff) == R*Qmult,
+    GMM_ASSERT1(gmm::vect_size(coeff) == nbdof*Qmult,
 		"Wrong size for coeff vector");
 
     gmm::clear(val);
     base_tensor Z; real_base_value(c, Z);
     
-    for (size_type j = 0; j < R; ++j) {
+    for (size_type j = 0; j < nbdof; ++j) {
       for (size_type q = 0; q < Qmult; ++q) {
-        typename gmm::linalg_traits<CVEC>::value_type co = 0.0;
-        co = coeff[j*Qmult+q];
+        typename gmm::linalg_traits<CVEC>::value_type co = coeff[j*Qmult+q];
         for (size_type r = 0; r < target_dim(); ++r)
-          val[r + q*target_dim()] += co * Z[j + r*R];
+          val[r + q*target_dim()] += co * Z[j + r*nbdof];
       }
     }
   }
@@ -744,41 +765,46 @@ namespace getfem {
   void virtual_fem::interpolation(const fem_interpolation_context& c,
                                   MAT &M, dim_type Qdim) const {
     size_type Qmult = size_type(Qdim) / target_dim();
-    size_type R = nb_dof(c.convex_num());
-    GMM_ASSERT1(gmm::mat_nrows(M) == Qdim && gmm::mat_ncols(M) == R*Qmult,
+    size_type nbdof = nb_dof(c.convex_num());
+    GMM_ASSERT1(gmm::mat_nrows(M) == Qdim && gmm::mat_ncols(M) == nbdof*Qmult,
                 "dimensions mismatch");
 
     gmm::clear(M);
     base_tensor Z; real_base_value(c, Z);
-    for (size_type j = 0; j < R; ++j) {
+    for (size_type j = 0; j < nbdof; ++j) {
       for (size_type q = 0; q < Qmult; ++q) {
         for (size_type r = 0; r < target_dim(); ++r)
-          M(r+q*target_dim(), j*Qmult+q) = Z[j + r*R];
+          M(r+q*target_dim(), j*Qmult+q) = Z[j + r*nbdof];
       }
     }
   }
 
 
+  // IN : coeff(Qmult,nb_dof)
+  // OUT: val(Qdim,N), Qdim=target_dim*Qmult
+  // AUX: t(nb_dof,target_dim,N)
   template<typename CVEC, typename VMAT>
   void virtual_fem::interpolation_grad(const fem_interpolation_context& c,
                                        const CVEC& coeff, VMAT &val,
                                        dim_type Qdim) const {
-    typedef typename gmm::linalg_traits<CVEC>::value_type T;
-    size_type Qmult = size_type(Qdim) / target_dim();
-    dim_type N = dim_type(c.N());
-    GMM_ASSERT1(gmm::mat_ncols(val) == N && gmm::mat_nrows(val) == Qdim,
+    size_type N = c.N();
+    size_type nbdof = nb_dof(c.convex_num());
+    size_type Qmult = gmm::vect_size(coeff) / nbdof;
+    GMM_ASSERT1(gmm::mat_ncols(val) == N &&
+                gmm::mat_nrows(val) == target_dim()*Qmult &&
+                gmm::vect_size(coeff) == nbdof*Qmult,
+                "dimensions mismatch");
+    GMM_ASSERT1(Qdim == target_dim()*Qmult, // Qdim seems to be superfluous input, could be removed in the future
                 "dimensions mismatch");
-
     base_tensor t;
-    size_type R = nb_dof(c.convex_num());
+    real_grad_base_value(c, t); // t(nbdof,target_dim,N)
 
     gmm::clear(val);
-    real_grad_base_value(c, t);
     for (size_type q = 0; q < Qmult; ++q) {
       base_tensor::const_iterator it = t.begin();
       for (size_type k = 0; k < N; ++k)
         for (size_type r = 0; r < target_dim(); ++r)
-          for (size_type j = 0; j < R; ++j, ++it)
+          for (size_type j = 0; j < nbdof; ++j, ++it)
             val(r + q*target_dim(), k) += coeff[j*Qmult+q] * (*it);
     }
   }
@@ -788,28 +814,68 @@ namespace getfem {
   void virtual_fem::interpolation_hess(const fem_interpolation_context& c,
                                        const CVEC& coeff, VMAT &val,
                                        dim_type Qdim) const {
-    typedef typename gmm::linalg_traits<CVEC>::value_type T;
     size_type Qmult = size_type(Qdim) / target_dim();
-    dim_type N = dim_type(c.N());
-    GMM_ASSERT1(gmm::mat_ncols(val) == gmm::size_type(N*N)
-                && gmm::mat_nrows(val) == Qdim, "dimensions mismatch");
+    size_type N = c.N();
+    GMM_ASSERT1(gmm::mat_ncols(val) == N*N &&
+                gmm::mat_nrows(val) == Qdim, "dimensions mismatch");
 
     base_tensor t;
-    size_type R = nb_dof(c.convex_num());
+    size_type nbdof = nb_dof(c.convex_num());
 
     gmm::clear(val);
     real_hess_base_value(c, t);
     for (size_type q = 0; q < Qmult; ++q) {
       base_tensor::const_iterator it = t.begin();
-      for (size_type k = 0; k < size_type(N*N); ++k)
+      for (size_type k = 0; k < N*N; ++k)
         for (size_type r = 0; r < target_dim(); ++r)
-          for (size_type j = 0; j < R; ++j, ++it)
+          for (size_type j = 0; j < nbdof; ++j, ++it)
             val(r + q*target_dim(), k) += coeff[j*Qmult+q] * (*it);
     }
   }
 
 
-  /* Functions allowing the add of a finite element method outwards
+  // IN : coeff(Qmult,nb_dof)
+  // OUT: val
+  // AUX: t(nb_dof,target_dim,N), Qmult*target_dim == N
+  template<typename CVEC>
+  void virtual_fem::interpolation_diverg
+    (const fem_interpolation_context& c, const CVEC& coeff,
+     typename gmm::linalg_traits<CVEC>::value_type &val) const {
+    size_type N = c.N();
+    size_type nbdof = nb_dof(c.convex_num());
+    size_type Qmult = gmm::vect_size(coeff) / nbdof;
+    GMM_ASSERT1(gmm::vect_size(coeff) == nbdof*Qmult , "dimensions mismatch");
+    GMM_ASSERT1(target_dim()*Qmult == N &&
+                (Qmult == 1 || target_dim() == 1),
+                "Dimensions mismatch. Divergence operator requires fem qdim equal to dim.");
+    base_tensor t;
+    real_grad_base_value(c, t); // t(nbdof,target_dim,N)
+    // for Qmult == 1 this is sub-optimal since it evaluates all (:,i,j)
+    // gradients instead of only the diagonal ones(:,i,i)
+
+    val = scalar_type(0);
+    base_tensor::const_iterator it = t.begin();
+    if (Qmult == 1)
+      for (size_type k = 0; k < N; ++k) {
+        if (k) it += (N*nbdof + 1);
+        for (size_type j = 0; j < nbdof; ++j) {
+          if (j) ++it;
+          val += coeff[j] * (*it);
+        }
+      }
+    else // if (target_dim() == 1)
+      for (size_type k = 0; k < N; ++k) {
+        if (k) ++it;
+        for (size_type j = 0; j < nbdof; ++j) {
+          if (j) ++it;
+          val += coeff[j*N+k] * (*it);
+        }
+      }
+  }
+
+
+
+  /* Functions allowing the add of a finite element method outside
      of getfem_fem.cc */
 
   typedef dal::naming_system<virtual_fem>::param_list fem_param_list;
