@@ -1,11 +1,11 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
- 
- Copyright (C) 1999-2012 Yves Renard
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 1999-2015 Yves Renard
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -17,7 +17,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
  As a special exception, you  may use  this file  as it is a part of a free
  software  library  without  restriction.  Specifically,  if   other  files
  instantiate  templates  or  use macros or inline functions from this file,
@@ -26,7 +26,7 @@
  to be covered  by the GNU Lesser General Public License.  This   exception
  does not  however  invalidate  any  other  reasons why the executable file
  might be covered by the GNU Lesser General Public License.
- 
+
 ===========================================================================*/
 
 /**@file getfem_mesh_fem.h
@@ -133,7 +133,7 @@ namespace getfem {
     value_type front(void) const { return *begin(); }
     value_type back(void) const { return *(--(end())); }
 
-    tab_scal_to_vect(void) {}
+    tab_scal_to_vect(void) : N(0) {}
     tab_scal_to_vect(const CONT &cc, dim_type n)
       : it(cc.begin()), ite(cc.end()), N(n) {}
     
@@ -165,9 +165,10 @@ namespace getfem {
                        /* of element option. (-1 = no automatic addition) */
     bool auto_add_elt_disc;
     scalar_type auto_add_elt_alpha;
-    dim_type Qdim; /* this is the "global" target_dim */
-    dim_type QdimM, QdimN; /* for matrix field with QdimM lines and QdimN */
-                           /* columnsQdimM * QdimN = Qdim.                */
+    dim_type Qdim;         /* this is the "total" target_dim. */
+    bgeot::multi_index mi; /* multi dimensions for matrix/tensor field. */ 
+    // dim_type QdimM, QdimN; /* for matrix field with QdimM lines and QdimN */
+    //                       /* columnsQdimM * QdimN = Qdim.                */
     std::vector<size_type> dof_partition;
     mutable gmm::uint64_type v_num_update, v_num;
     bool use_reduction;    /* A reduction matrix is applied or not.       */
@@ -293,27 +294,57 @@ namespace getfem {
 	linked_mesh().dim().
      */
     virtual dim_type get_qdim() const { return Qdim; }
+    virtual const bgeot::multi_index &get_qdims() const { return mi; }
+
     /** Change the Q dimension */
     virtual void set_qdim(dim_type q) {
-      if (q != Qdim || q != QdimM) {
-	QdimM = Qdim = q; QdimN = 1;
+      if (q != Qdim || mi.size() != 1) {
+        mi.resize(1); mi[0] = q; Qdim = q;
+        dof_enumeration_made = false; touch(); v_num = act_counter();
+      }
+    }
+
+    /** Set the dimension for a matrix field. */
+    virtual void set_qdim(dim_type M, dim_type N) {
+      if (mi.size() != 2 || mi[0] != M || mi[1] != N) {
+        mi.resize(2); mi[0] = M; mi[1] = N; Qdim = dim_type(M*N);
 	dof_enumeration_made = false; touch(); v_num = act_counter();
       }
     }
 
-    /** Set the dimension for a matrix field. The Q dimension will
-	always be the product of get_qdim_m and get_qdim_n */
-    virtual void set_qdim_mn(dim_type M, dim_type N) {
-      if (M != QdimM || N != QdimN) {
-	QdimM = M; QdimN = N; Qdim = dim_type(N*M);
+    /** Set the dimension for a fourth order tensor field. */
+    virtual void set_qdim(dim_type M, dim_type N, dim_type O, dim_type P) {
+      if (mi.size() != 4 || mi[0] != M || mi[1] != N || mi[2] != O
+          || mi[3] != P) {
+        mi.resize(4); mi[0] = M; mi[1] = N; mi[2] = O; mi[3] = P;
+        Qdim = dim_type(M*N*O*P);
 	dof_enumeration_made = false; touch(); v_num = act_counter();
       }
     }
 
+    /** Set the dimension for an arbitrary order tensor field. */
+    virtual void set_qdim(const bgeot::multi_index &mii) {
+      GMM_ASSERT1(mii.size() < 7,
+                  "Tensor field are taken into account up to order 6.");
+      GMM_ASSERT1(mi.size(), "Wrong sizes");
+      if (!(mi.is_equal(mii))) {
+        mi = mii;
+        Qdim = dim_type(1);
+        for (size_type i = 0; i < mi.size(); ++i)
+          Qdim = dim_type(Qdim*mi[i]);
+        GMM_ASSERT1(Qdim, "Wrong sizes");
+        dof_enumeration_made = false; touch(); v_num = act_counter();
+      }
+    }
+    
+
+    void set_qdim_mn(dim_type M, dim_type N) IS_DEPRECATED
+    { set_qdim(M,N); }
+    
     /** for matrix fields, return the number of rows. */
-    dim_type get_qdim_m() const { return QdimM; }
+    dim_type get_qdim_m() const IS_DEPRECATED { return dim_type(mi[0]); }
     /** for matrix fields, return the number of columns. */
-    dim_type get_qdim_n() const { return QdimN; }
+    dim_type get_qdim_n() const IS_DEPRECATED { return dim_type(mi[1]); }
      
 
     /** Set the finite element method of a convex.
@@ -422,6 +453,7 @@ namespace getfem {
     */
     virtual size_type nb_basic_dof_of_face_of_element(size_type cv,
 					      short_type f) const {
+      context_check(); if (!dof_enumeration_made) enumerate_dof();
       pfem pf = f_elems[cv];
       return dof_structure.structure_of_convex(cv)->nb_points_of_face(f)
 	* Qdim / pf->target_dim();
@@ -432,8 +464,10 @@ namespace getfem {
     /** Return the number of  degrees of freedom attached to a given convex.
 	@param cv the convex number.
     */
-    virtual size_type nb_basic_dof_of_element(size_type cv) const
-    { pfem pf = f_elems[cv]; return pf->nb_dof(cv) * Qdim / pf->target_dim(); }
+    virtual size_type nb_basic_dof_of_element(size_type cv) const {
+      context_check(); if (!dof_enumeration_made) enumerate_dof();
+      pfem pf = f_elems[cv]; return pf->nb_dof(cv) * Qdim / pf->target_dim();
+    }
     size_type nb_dof_of_element(size_type cv) const IS_DEPRECATED
     { return nb_basic_dof_of_element(cv); }
     
@@ -484,7 +518,7 @@ namespace getfem {
     virtual void get_global_dof_index(std::vector<size_type> &ind) const;
     /** Renumber the degrees of freedom. You should not have
      * to call this function, as it is done automatically */
-    void enumerate_dof(void) const;
+    virtual void enumerate_dof(void) const;
 
 #if GETFEM_PARA_LEVEL > 1
     void enumerate_dof_para(void)const;
@@ -584,6 +618,32 @@ namespace getfem {
   const mesh_fem &dummy_mesh_fem(void);
 
 
+  /** Given a mesh_fem @param mf and a vector @param vec of size equal to
+   *  mf.nb_basic_dof(), the output vector @param coeff will contain the
+   *  values of @param vec corresponding to the basic dofs of element
+   *  @param cv . The size of @param coeff is adjusted if necessary.
+   */
+  template <typename VEC1, typename VEC2>
+  void slice_vector_on_basic_dof_of_element(const mesh_fem &mf,
+                                            const VEC1 &vec,
+                                            size_type cv, VEC2 &coeff) {
+    size_type nbdof = mf.nb_basic_dof();
+    size_type qmult = gmm::vect_size(vec) / nbdof;
+    GMM_ASSERT1(gmm::vect_size(vec) == qmult * nbdof, "Bad dof vector size");
+    size_type cvnbdof = mf.nb_basic_dof_of_element(cv);
+    gmm::resize(coeff, cvnbdof*qmult);
+    mesh_fem::ind_dof_ct::const_iterator
+      itdof = mf.ind_basic_dof_of_element(cv).begin();
+    if (qmult == 1) {
+      for (size_type k = 0; k < cvnbdof; ++k, ++itdof)
+          coeff[k] = vec[*itdof];
+    } else {
+      for (size_type k = 0; k < cvnbdof; ++k, ++itdof)
+        for (size_type l = 0; l < qmult; ++l)
+          coeff[l+k*qmult] = vec[l+(*itdof)*qmult];
+    }
+  }
+
 }  /* end of namespace getfem.                                             */
 
 
