@@ -1,11 +1,11 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
- 
- Copyright (C) 2000-2012 Yves Renard, Julien Pommier
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2000-2015 Yves Renard, Julien Pommier
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -17,7 +17,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
  As a special exception, you  may use  this file  as it is a part of a free
  software  library  without  restriction.  Specifically,  if   other  files
  instantiate  templates  or  use macros or inline functions from this file,
@@ -26,7 +26,7 @@
  to be covered  by the GNU Lesser General Public License.  This   exception
  does not  however  invalidate  any  other  reasons why the executable file
  might be covered by the GNU Lesser General Public License.
- 
+
 ===========================================================================*/
 
 /**@file getfem_nonlinear_elasticity.h
@@ -38,11 +38,11 @@
 #ifndef GETFEM_NONLINEAR_ELASTICITY_H__
 #define GETFEM_NONLINEAR_ELASTICITY_H__
 
-#include "getfem_modeling.h"
 #include "getfem_models.h"
 #include "getfem_assembling_tensors.h"
 #include "getfem_derivatives.h"
 #include "getfem_interpolation.h"
+#include "getfem_generic_assembly.h"
 #include "gmm/gmm_inoutput.h"
 
 namespace getfem {
@@ -125,7 +125,6 @@ namespace getfem {
       caracterized by Ex, Ey, vYX and G, with optional orthotropic prestresses.
       due to Jean-Yves Heddebaut <jyhed@alpino.be>
   */
-
   struct membrane_elastic_law : 
     public abstract_hyperelastic_law {
     virtual scalar_type strain_energy(const base_matrix &E,
@@ -140,19 +139,41 @@ namespace getfem {
 
 
   /** Mooney-Rivlin hyperelastic law 
-      
-      To be used for incompressible problems (with getfem::mdbrick_nonlinear_incomp).
+
+      To be used for compressible and incompressible problems.
+      Following combinations are possible:
+        not compressible, not neohookean (default): 2 parameters (C1,C2)
+        not compressible, neohookean: 1 parameter (C1)
+        compressible, not neohookean: 3 parameters (C1,C2,D1)
+        compressible, neohookean: 2 parameters (C1,D1)
   */
   struct Mooney_Rivlin_hyperelastic_law : public abstract_hyperelastic_law {
+    const bool compressible, neohookean;
     virtual scalar_type strain_energy(const base_matrix &E,
 				      const base_vector &params, scalar_type det_trans) const;
     virtual void sigma(const base_matrix &E, base_matrix &result,
 		       const base_vector &params, scalar_type det_trans) const;
     virtual void grad_sigma(const base_matrix &E, base_tensor &result,
 			    const base_vector &params, scalar_type det_trans) const;
-    Mooney_Rivlin_hyperelastic_law(void);
+    Mooney_Rivlin_hyperelastic_law(bool compressible_=false,
+                                   bool neohookean_=false);
   };
 
+  /** Neo-Hookean hyperelastic law variants
+
+      To be used for compressible problems.
+      For incompressible ones Mooney-Rivlin hyperelastic law can be used.
+  */
+  struct Neo_Hookean_hyperelastic_law : public abstract_hyperelastic_law {
+    const bool bonet;
+    virtual scalar_type strain_energy(const base_matrix &E,
+                                      const base_vector &params, scalar_type det_trans) const;
+    virtual void sigma(const base_matrix &E, base_matrix &result,
+                       const base_vector &params, scalar_type det_trans) const;
+    virtual void grad_sigma(const base_matrix &E, base_tensor &result,
+                            const base_vector &params, scalar_type det_trans) const;
+    Neo_Hookean_hyperelastic_law(bool bonet_=true);
+  };
 
   /** Blatz_Ko hyperelastic law 
       
@@ -169,12 +190,11 @@ namespace getfem {
   };
 
 
-  /** Ciarlet-Geymonat hyperelastic law ( @f$ W=~_1i_1(L) + \frac{~}{2}i_2(L) + 8ci_3(L) - \frac{~_1}{2} \textrm{log}~\textrm{det}~C @f$ )
-      
+  /** Ciarlet-Geymonat hyperelastic law
    */
   struct Ciarlet_Geymonat_hyperelastic_law : public abstract_hyperelastic_law {
-    // parameters are lambda=params[0], mu=params[1], gamma'(1)=params[2]
-    // The parameter gamma'(1) has to verify gamma'(1) in ]max{-lambda/2-mu, -2mu}, -mu[
+    // parameters are lambda=params[0], mu=params[1], a=params[2]
+    // The parameter a has to verify a in ]0, mu/2[
     virtual scalar_type strain_energy(const base_matrix &E,
 				      const base_vector &params, scalar_type det_trans) const;
     virtual void sigma(const base_matrix &E, base_matrix &result,
@@ -236,14 +256,12 @@ namespace getfem {
 	gmm::copy(PARAMS, params);
     }
 
-    const bgeot::multi_index &sizes() const {  return sizes_; }
+    const bgeot::multi_index &sizes(size_type) const {  return sizes_; }
+
     virtual void compute(getfem::fem_interpolation_context& ctx,
 			 bgeot::base_tensor &t) {
       size_type cv = ctx.convex_num();
-      // cout << "compute cv " << cv << " pt " << ctx.xref() << endl;
-      coeff.resize(mf.nb_basic_dof_of_element(cv));
-      gmm::copy(gmm::sub_vector
-		(U, gmm::sub_index(mf.ind_basic_dof_of_element(cv))), coeff);
+      slice_vector_on_basic_dof_of_element(mf, U, cv, coeff);
       ctx.pf()->interpolation_grad(ctx, coeff, gradU, mf.get_qdim());
       
       for (unsigned int alpha = 0; alpha < N; ++alpha)
@@ -430,8 +448,6 @@ namespace getfem {
   /*		Mixed nonlinear incompressibility assembly procedure      */
   /* ******************************************************************** */
 
-# define MDBRICK_NONLINEAR_INCOMP 964552
-
   template<typename VECT1> class incomp_nonlinear_term 
     : public getfem::nonlinear_elem_term {
 
@@ -454,14 +470,12 @@ namespace getfem {
       mf.extend_vector(U_, U);
     }
 
-    const bgeot::multi_index &sizes() const { return sizes_; }
+    const bgeot::multi_index &sizes(size_type) const { return sizes_; }
 
     virtual void compute(getfem::fem_interpolation_context& ctx,
 			 bgeot::base_tensor &t) {
       size_type cv = ctx.convex_num();
-      coeff.resize(mf.nb_basic_dof_of_element(cv));
-      gmm::copy(gmm::sub_vector
-		(U, gmm::sub_index(mf.ind_basic_dof_of_element(cv))), coeff);
+      slice_vector_on_basic_dof_of_element(mf, U, cv, coeff);
       ctx.pf()->interpolation_grad(ctx, coeff, gradPhi, mf.get_qdim());
       gmm::add(gmm::identity_matrix(), gradPhi);
       scalar_type det = gmm::lu_inverse(gradPhi);
@@ -567,7 +581,8 @@ namespace getfem {
 
   /** Add a nonlinear (large strain) elasticity term to the model with
       respect to the variable
-      `varname`. Note that the constitutive law is described by `AHL` which
+      `varname` (deprecated brick, use add_finite_strain_elaticity instead).
+      Note that the constitutive law is described by `AHL` which
       should not be freed since the model is used. `dataname` described the
       parameters of the constitutive laws. It could be a vector of value
       of length the number of parameter of the constitutive law or a vector
@@ -608,7 +623,6 @@ namespace getfem {
     gmm::copy(VMM, VM);
   }
   
-
   /** Add a nonlinear incompressibility term (for large strain elasticity)
       to the model with respect to the variable
       `varname` (the displacement) and `multname` (the pressure).
@@ -617,267 +631,41 @@ namespace getfem {
   (model &md, const mesh_im &mim, const std::string &varname,
    const std::string &multname, size_type region = size_type(-1));
 
-
-
-
-
-
-
-
-
-
-
-
   //===========================================================================
-  //
-  //  Bricks for the old brick system (DEPRECATED)
-  //
+  //  High-level generic assembly bricks
   //===========================================================================
 
-
-# define MDBRICK_NONLINEAR_ELASTICITY 821357
-
-  /** Non-linear elasticity brick  ( @f$ \int (I+\nabla u)\hat{\hat{\sigma}}:\nabla v = l(v)  @f$ ).
-      
-      @f$ \hat{\hat{\sigma}} @f$ is known as the second Piola-Kirchhoff stress tensor, and is given by \f[ \hat{\hat{\sigma}} = -\frac{\partial}{\partial L}W(L) \f],
-      with @f$W@f$ the strain energy of the material, and @f$L=\frac{1}{2}\left(\nabla u^t\nabla u + \nabla u^t + \nabla u\right)@f$ is the Green-Lagrange strain tensor.
-
-      This brick handle the computation of the tangent matrix and the
-      right hand side for large strain problems, with hyperelastic
-      material laws.
-  
-      @ingroup bricks
+  /** Add a finite strain elasticity brick
+      to the model with respect to the variable
+      `varname` (the displacement).
+      For 2D meshes, switch automatically to plane strain elasticity.
+      High-level generic assembly version.
   */
-    template<typename MODEL_STATE = standard_model_state>
-    class mdbrick_nonlinear_elasticity : public mdbrick_abstract<MODEL_STATE> {
-
-      TYPEDEF_MODEL_STATE_TYPES;
-
-      const abstract_hyperelastic_law &AHL;
-      const mesh_im &mim;
-      const mesh_fem &mf_u;
-      mdbrick_parameter<VECTOR> PARAMS_;
-
-      virtual void proper_update(void) {}
-
-    public :
-
-      virtual void do_compute_tangent_matrix(MODEL_STATE &MS, size_type i0,
-					     size_type) {
-	gmm::sub_interval SUBI(i0, mf_u.nb_dof());
-	gmm::clear(gmm::sub_matrix(MS.tangent_matrix(), SUBI));
-	asm_nonlinear_elasticity_tangent_matrix
-	  (gmm::sub_matrix(MS.tangent_matrix(), SUBI), mim, mf_u,
-	   gmm::sub_vector(MS.state(), SUBI), &(params().mf()), params().get(),
-	   AHL, this->mf_u.linked_mesh().get_mpi_region());
-      }
-      virtual void do_compute_residual(MODEL_STATE &MS, size_type i0, size_type) {
-	gmm::sub_interval SUBI(i0, mf_u.nb_dof());
-	gmm::clear(gmm::sub_vector(MS.residual(), SUBI));
-	asm_nonlinear_elasticity_rhs(gmm::sub_vector(MS.residual(), SUBI), mim,
-				     mf_u, gmm::sub_vector(MS.state(), SUBI), 
-				     &(params().mf()), params().get(), AHL,
-				     this->mf_u.linked_mesh().get_mpi_region());
-      }
-
-      mdbrick_parameter<VECTOR> &params() { 
-	PARAMS_.reshape(AHL.nb_params()); return PARAMS_; 
-      }
-
-      SUBVECTOR get_solution(MODEL_STATE &MS) {
-	gmm::sub_interval SUBU(this->first_index(), mf_u.nb_dof());
-	return gmm::sub_vector(MS.state(), SUBU);
-      }
+  size_type add_finite_strain_elasticity_brick
+  (model &md, const mesh_im &mim, const std::string &varname,
+   const std::string &lawname, const std::string &params,
+   size_type region = size_type(-1));
 
 
-
-      /* modified by Jean-Yves Heddebaut <jyhed@alpino.be> sep08, to handle
-	 cases where mesh dim <> fem qdim.
-
-	 For the membrane case, the cauchy stresses are calculated starting
-	 from the 2X2 lagrange tensor.
-	 To compute the jacobian in case of 2D mesh with 3D displacements,
-	 a 3th column is added to gradphi, expressing that the  unit vector
-	 perpendicular to the reference surface eZ' (0,0,1), is transformed
-	 in a  unit vector perpendicular to the deformed surface. (eZ is the
-	 vector eX X eY, with eX and eY = images of eX' and eY')
-	 (to be validated!)
-      */
-
-      template <class VECTVM>
-      void compute_Von_Mises_or_Tresca(MODEL_STATE &MS, const mesh_fem &mf_vm,
-				       VECTVM &VM, bool tresca) {
-	unsigned N = unsigned(mf_u.linked_mesh().dim());
-	unsigned NP = unsigned(AHL.nb_params()), NFem = mf_u.get_qdim();
-	VECTOR GRAD(mf_vm.nb_dof()*NFem*N), PARAMS(mf_vm.nb_dof()*NP);      
-	interpolation(PARAMS_.mf(), mf_vm, PARAMS_.get(), PARAMS);
-	compute_gradient(mf_u, mf_vm, get_solution(MS), GRAD);
-	GMM_ASSERT1(gmm::vect_size(VM) == mf_vm.nb_dof(),
-		    "The vector has not the good size");
-	base_matrix E(N, N), gradphi(NFem,N),gradphit(N,NFem), Id(N, N),
-	  sigmahathat(N,N),aux(NFem,N), sigma(NFem,NFem),
-	  IdNFem(NFem, NFem);
-	base_vector p(NP);
-	base_vector eig(NFem);
-	base_vector ez(NFem);	// vector normal at deformed surface, (ex X ey)
-	double normEz(0);		//norm of ez
-	gmm::copy(gmm::identity_matrix(), Id);
-	gmm::copy(gmm::identity_matrix(), IdNFem);
-	for (size_type i = 0; i < mf_vm.nb_dof(); ++i) {
-	  gmm::resize(gradphi,NFem,N);
-	  std::copy(GRAD.begin()+i*NFem*N, GRAD.begin()+(i+1)*NFem*N,
-		    gradphit.begin());
-	  gmm::copy(gmm::transposed(gradphit),gradphi);
-	  for (unsigned int alpha = 0; alpha <N; ++alpha)
-	    gradphi(alpha, alpha)+=1;
-	  gmm::mult(gmm::transposed(gradphi), gradphi, E);
-	  gmm::add(gmm::scaled(Id, -scalar_type(1)), E);
-	  gmm::scale(E, scalar_type(1)/scalar_type(2));
-	  gmm::copy(gmm::sub_vector(PARAMS, gmm::sub_interval(i*NP,NP)), p);
-	  AHL.sigma(E, sigmahathat, p, scalar_type(1));
-	  if (NFem == 3 && N == 2) {
-	    //jyh : compute ez, normal on deformed surface
-	    for (unsigned int l = 0; l <NFem; ++l)  {
-	      ez[l]=0;
-	      for (unsigned int m = 0; m <NFem; ++m) 
-		for (unsigned int n = 0; n <NFem; ++n){
-		  ez[l]+=levi_civita(l,m,n)*gradphi(m,0)*gradphi(n,1);
-		}
-	      normEz= gmm::vect_norm2(ez);
-	    }
-	    //jyh : end compute ez
-	  }
-	  gmm::mult(gradphi, sigmahathat, aux);
-	  gmm::mult(aux, gmm::transposed(gradphi), sigma);
-	
-	  /* jyh : complete gradphi for virtual 3rd dim (perpendicular to
-	     deformed surface, same thickness) */
-	  if (NFem == 3 && N == 2) {
-	    gmm::resize(gradphi,NFem,NFem);
-	    for (unsigned int ll = 0; ll <NFem; ++ll) 
-	      for (unsigned int ii = 0; ii <NFem; ++ii) 
-		for (unsigned int jj = 0; jj <NFem; ++jj) 
-		  gradphi(ll,2)+=(levi_civita(ll,ii,jj)*gradphi(ii,0)
-				  *gradphi(jj,1))/normEz;
-	    //jyh : end complete graphi
-	  }
-	
-	  gmm::scale(sigma, scalar_type(1) / gmm::lu_det(gradphi));
-	
-	  if (!tresca) {
-	    /* von mises: norm(deviator(sigma)) */
-	    gmm::add(gmm::scaled(IdNFem, -gmm::mat_trace(sigma) / NFem), sigma);
-	  
-	    //jyh : von mises stress=sqrt(3/2)* norm(sigma) ?
-	    VM[i] = sqrt(3.0/2)*gmm::mat_euclidean_norm(sigma);
-	  } else {
-	    /* else compute the tresca criterion */
-	    //jyh : to be adapted for membrane if necessary
-	    gmm::symmetric_qr_algorithm(sigma, eig);
-	    std::sort(eig.begin(), eig.end());
-	    VM[i] = eig.back() - eig.front();
-	  }
-	}
-      }
-
-      mdbrick_nonlinear_elasticity(const abstract_hyperelastic_law &AHL_,
-				   const mesh_im &mim_,
-				   const mesh_fem &mf_u_,
-				   const VECTOR &PARAMS)
-	: AHL(AHL_), mim(mim_), mf_u(mf_u_),
-	  PARAMS_("params", mf_u.linked_mesh(), this) {
-	params().set(PARAMS);
-	this->add_proper_mesh_fem(mf_u, MDBRICK_NONLINEAR_ELASTICITY);
-	this->add_proper_mesh_im(mim);
-	this->proper_is_linear_ = false;
-	this->proper_is_coercive_ = this->proper_is_symmetric_ = true;
-	this->force_update();
-      }
-    };
-
-
-  /* ******************************************************************** */
-  /*		Mixed nonlinear incompressible condition brick.           */
-  /* ******************************************************************** */
-
-  /** Incompressible non-linear elasticity brick.
-      @ingroup bricks
+  /** Add a finite strain incompressibility term (for large strain elasticity)
+      to the model with respect to the variable
+      `varname` (the displacement) and `multname` (the pressure).
+      High-level generic assembly version.
   */
-  template<typename MODEL_STATE = standard_model_state>
-  class mdbrick_nonlinear_incomp : public mdbrick_abstract<MODEL_STATE>  {
-    
-    TYPEDEF_MODEL_STATE_TYPES;
-   
-    mdbrick_abstract<MODEL_STATE> &sub_problem;
-    const mesh_fem &mf_p;
-    size_type num_fem;
-
-    virtual void proper_update(void) {
-      this->proper_mixed_variables.clear();
-      this->proper_mixed_variables.add(sub_problem.nb_dof(), mf_p.nb_dof());
-    }
-
-  public :
-
-    SUBVECTOR get_pressure(MODEL_STATE &MS) {
-      gmm::sub_interval SUBU(this->first_index() + sub_problem.nb_dof(),
-			     mf_p.nb_dof());
-      return gmm::sub_vector(MS.state(), SUBU);
-    }
-    
-    template<typename MATRIX> void get_B(MODEL_STATE &MS, MATRIX B) {
-      const mesh_fem &mf_u = *(this->mesh_fems[num_fem]);
-      gmm::sub_interval SUBI(this->first_index()+sub_problem.nb_dof(), mf_p.nb_dof()); /* P */
-      gmm::sub_interval SUBJ(this->first_index()+this->mesh_fem_positions[num_fem], mf_u.nb_dof());           /* U */
-      gmm::copy(gmm::sub_matrix(MS.tangent_matrix(), SUBJ, SUBI), B);
-    }
-
-
-    virtual void do_compute_tangent_matrix(MODEL_STATE &MS, size_type i0,
-					   size_type) {
-      const mesh_fem &mf_u = *(this->mesh_fems[num_fem]);
-      size_type i1 = this->mesh_fem_positions[num_fem];
-      gmm::sub_interval SUBI(i0+sub_problem.nb_dof(), mf_p.nb_dof()); /* P */
-      gmm::sub_interval SUBJ(i0+i1, mf_u.nb_dof());           /* U */
-      T_MATRIX B(mf_u.nb_dof(), mf_p.nb_dof());
-      asm_nonlinear_incomp_tangent_matrix(gmm::sub_matrix(MS.tangent_matrix(),
-							  SUBJ, SUBJ), B,
-					  *(this->mesh_ims[0]), mf_u, mf_p, 
-					  gmm::sub_vector(MS.state(), SUBJ), 
-					  gmm::sub_vector(MS.state(), SUBI),
-					  mf_u.linked_mesh().get_mpi_region());
-      gmm::copy(B, gmm::sub_matrix(MS.tangent_matrix(), SUBJ, SUBI));
-      gmm::copy(gmm::transposed(B),
-		gmm::sub_matrix(MS.tangent_matrix(), SUBI, SUBJ));
-      gmm::clear(gmm::sub_matrix(MS.tangent_matrix(), SUBI, SUBI));
-    }
-
-    virtual void do_compute_residual(MODEL_STATE &MS, size_type i0, size_type) {
-      const mesh_fem &mf_u = *(this->mesh_fems[num_fem]);
-      size_type i1 = this->mesh_fem_positions[num_fem];
-      gmm::sub_interval SUBI(i0 + sub_problem.nb_dof(), mf_p.nb_dof());
-      gmm::sub_interval SUBJ(i0+i1, mf_u.nb_dof());
-      gmm::clear(gmm::sub_vector(MS.residual(), SUBI));
-      asm_nonlinear_incomp_rhs(gmm::sub_vector(MS.residual(), SUBJ),
-			       gmm::sub_vector(MS.residual(), SUBI),
-			       *(this->mesh_ims[0]), mf_u, mf_p, 
-			       gmm::sub_vector(MS.state(), SUBJ),
-			       gmm::sub_vector(MS.state(), SUBI),
-			       mf_u.linked_mesh().get_mpi_region());
-    }
-
-    mdbrick_nonlinear_incomp(mdbrick_abstract<MODEL_STATE> &problem,
-			     const mesh_fem &mf_p_, size_type num_fem_=0)
-      : sub_problem(problem), mf_p(mf_p_), num_fem(num_fem_) {
-      this->add_proper_mesh_fem(mf_p, MDBRICK_NONLINEAR_INCOMP);
-      this->add_sub_brick(sub_problem);
-      this->proper_is_linear_ = this->proper_is_coercive_ = false;
-      this->proper_is_symmetric_ = true;
-      this->force_update();
-    }
-  };
-
+  size_type add_finite_strain_incompressibility_brick
+  (model &md, const mesh_im &mim, const std::string &varname,
+   const std::string &multname, size_type region = size_type(-1));
 
+  /**
+     Interpolate the Von-Mises stress of a field `varname`
+     with respect to the nonlinear elasticity constitutive law `lawname`
+     with parameters `params` (only valid in 3D).
+  */
+  void finite_strain_elasticity_Von_Mises
+  (model &md, const std::string &varname, const std::string &lawname,
+   const std::string &params, const mesh_fem &mf_vm,
+   model_real_plain_vector &VM,
+   const mesh_region &rg=mesh_region::all_convexes());
 
 }  /* end of namespace getfem.                                             */
 
