@@ -1,10 +1,10 @@
 /*===========================================================================
- 
- Copyright (C) 2005-2012 Yves Renard
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2005-2015 Yves Renard
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -16,7 +16,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
 ===========================================================================*/
 
 #include "getfem/getfem_mesh_im_level_set.h"
@@ -187,8 +187,8 @@ namespace getfem {
       scalar_type d1 = (mesherls0[i])(P);
       scalar_type d2 = (sec ? (mesherls1[i])(P) : -1);
       if (d1 < 0 && d2 < 0) ev.in.add(i);
-      if ((integrate_where & INTEGRATE_OUTSIDE) /*&& !sec*/)
-	ev.in[i].flip();
+      // if ((integrate_where & INTEGRATE_OUTSIDE) /*&& !sec*/)
+      //	ev.in[i].flip();
 
       if (gmm::abs(d1) < 1e-7 && d2 < 1e-7) 
 	ev.bin.add(i);
@@ -196,12 +196,17 @@ namespace getfem {
     
 
     bool2 r;
-    if (ls_csg_description.size()) 
+    if (ls_csg_description.size())
       r = ev.is_in(ls_csg_description.c_str());
     else {
       r.in  = (ev.in.card() == mls->nb_level_sets());
       r.bin = (ev.bin.card() >= 1 && ev.in.card() >= mls->nb_level_sets()-1);
     }
+
+    if (integrate_where & INTEGRATE_OUTSIDE) r.in = !(r.in);
+    
+
+
     /*bool2 r2 = is_point_in_selected_area2(mesherls0,mesherls1,P);
     if (r2.in != r.in || r2.bin != r.bin) {
       cerr << "ev.in = " << ev.in << ", bin=" << ev.bin<<"\n";
@@ -252,6 +257,7 @@ namespace getfem {
        "Base integration method for quasi polar integration not convenient");
 
     approx_integration *new_approx = new approx_integration(pgt->convex_ref());
+    new_approx->set_built_on_the_fly();
     base_matrix KK(n,n), CS(n,n);
     base_matrix pc(pgt2->nb_points(), n);
     std::vector<size_type> ptsing;
@@ -261,6 +267,8 @@ namespace getfem {
     for (dal::bv_visitor i(msh.convex_index()); !i.finished(); ++i) {
       papprox_integration pai = regular_simplex_pim->approx_method();
       
+      GMM_ASSERT1(regular_simplex_pim->structure() == bgeot::simplex_structure(n), "Base integration method should be defined on a simplex of same dimension than the mesh");
+      
       if ((integrate_where != INTEGRATE_ALL) &&
 	  !convexes_arein[i]) continue;
       
@@ -433,52 +441,6 @@ namespace getfem {
   }
 
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
   void mesh_im_cross_level_set::update_from_context(void) const
   { is_adapted = false; }
 
@@ -584,6 +546,7 @@ namespace getfem {
     dim_type n = pgt->dim();
 
     approx_integration *new_approx = new approx_integration(pgt->convex_ref());
+    new_approx->set_built_on_the_fly();
     base_matrix KK(n,n), CS(n,n);
     base_matrix pc(pgt2->nb_points(), n);
     std::vector<size_type> ptsing;
