@@ -1,10 +1,10 @@
 /*===========================================================================
- 
- Copyright (C) 2002-2012 Yves Renard, Julien Pommier.
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2002-2015 Yves Renard, Julien Pommier.
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -16,7 +16,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
 ===========================================================================*/
 
 /**
@@ -26,7 +26,7 @@
    the analytical solution.
 
    This program is used to check that getfem++ is working. This is also 
-   a good example of use of Getfem++.
+   a good example of use of GetFEM++.
 
    @see laplacian.cc
    @see nonlinear_elastostatic.cc
@@ -46,7 +46,7 @@ using std::endl; using std::cout; using std::cerr;
 using std::ends; using std::cin;
 
 
-/* some Getfem++ types that we will be using */
+/* some GetFEM++ types that we will be using */
 using bgeot::base_small_vector; /* special class for small (dim<16) vectors */
 using bgeot::base_node;  /* geometrical nodes(derived from base_small_vector)*/
 using bgeot::scalar_type; /* = double */
@@ -212,7 +212,7 @@ struct elastostatic_problem {
 
   scalar_type residual;       /* max residual for iterative solvers          */
   bool mixed_pressure, refine;
-  getfem::constraints_type dirichlet_version;
+  size_type dirichlet_version;
 
   std::string datafilename;
   bgeot::md_param PARAM;
@@ -277,7 +277,7 @@ void elastostatic_problem::init(void) {
 #endif
 
   dirichlet_version
-    = getfem::constraints_type(PARAM.int_value("DIRICHLET_VERSION",
+    = size_type(PARAM.int_value("DIRICHLET_VERSION",
 					       "Dirichlet version"));
   datafilename = PARAM.string_value("ROOTFILENAME","Base name of data files.");
   scalar_type FT = PARAM.real_value("FT", "parameter for exact solution");
@@ -374,10 +374,10 @@ void elastostatic_problem::compute_error(plain_vector &U) {
 	 << "H1 error = " << h1 << endl
 	 << "Linfty error = " << gmm::vect_norminf(V) << endl;
   
-  getfem::vtk_export exp(datafilename + "_err.vtk",
-			 PARAM.int_value("VTK_EXPORT")==1);
-  exp.exporting(mf_rhs); 
-  exp.write_point_data(mf_rhs, V, "elastostatic_displacement");
+/*   getfem::vtk_export exp(datafilename + "_err.vtk", */
+/* 			 PARAM.int_value("VTK_EXPORT")==1); */
+/*   exp.exporting(mf_rhs);  */
+/*   exp.write_point_data(mf_rhs, V, "elastostatic_displacement"); */
 
   mf_rhs.set_qdim(1);
 }
